import json
from copy import deepcopy

import pytest

from reptor.lib.exceptions import IncompatibleDesignException
from reptor.models.Finding import (
    Finding,
    FindingData,
    FindingDataField,
    FindingDataRaw,
    FindingRaw,
)
from reptor.models.ProjectDesign import ProjectDesign


class TestFindingModelParsing:
    example_design_with_finding_fields_only = """
    {"finding_fields":{"cvss":{"type":"cvss","label":"CVSS","origin":"core","default":"n/a","required":true},"title":{"type":"string","label":"Title","origin":"core","default":"TODO: Finding Title","required":true,"spellcheck":true},"date_field":{"type":"date","label":"Date Field","origin":"custom","default":null,"required":true},"enum_field":{"type":"enum","label":"Enum Field","origin":"custom","choices":[{"label":"Enum Value 1","value":"enum_val_1"},{"label":"Enum Value 2","value":"enum_val_2"},{"label":"Enum Value 3","value":"enum_val_"}],"default":null,"required":true},"list_field":{"type":"list","items":{"type":"object","label":"","origin":"custom","properties":{"enum_in_object":{"type":"enum","label":"Enum in Object","origin":"custom","choices":[{"label":"Enum in Obj 1","value":"enum_in_obj_1"},{"label":"Enum in Obj 2","value":"enum_in_obj_2"},{"label":"Enum in Obj 3","value":"enum_in_obj_3"}],"default":null,"required":true}}},"label":"List Field","origin":"custom","required":true},"user_field":{"type":"user","label":"User Field","origin":"custom","required":true},"number_field":{"type":"number","label":"Number Field","origin":"custom","default":null,"required":true},"object_field":{"type":"object","label":"Object Field","origin":"custom","properties":{"list_in_object":{"type":"list","items":{"type":"string","label":"","origin":"custom","default":null,"required":true,"spellcheck":false},"label":"List in Object","origin":"custom","required":true}}},"boolean_field":{"type":"boolean","label":"Boolean Field","origin":"custom","default":null},"combobox_field":{"type":"combobox","label":"Combobox Field","origin":"custom","default":null,"required":true,"suggestions":["Combobox Value 1","Combobox Value 2","Combobox Value 3"]},"markdown_field":{"type":"markdown","label":"Markdown Field","origin":"custom","default":null,"required":true}}}"""

    example_finding = """
    {
        "id": "d3658ee5-2d43-40f6-9b97-1b98480afe78",
        "created": "2023-07-16T20:07:36.213385Z",
        "updated": "2023-07-16T20:08:59.749591Z",
        "project": "a4b4b630-fc78-452d-b348-d362b69c2449",
        "project_type": "2970149f-e11d-420a-8a5d-25b5fda14e33",
        "language": "en-US",
        "lock_info": null,
        "template": null,
        "assignee": {
            "id": "788dcb76-9928-46fc-87ba-7043708f1bc0",
            "username": "demo-fTaIO4fj",
            "name": "",
            "title_before": null,
            "first_name": "",
            "middle_name": null,
            "last_name": "",
            "title_after": null,
            "is_active": true
        },
        "status": "in-progress",
        "data": {
            "cvss": "n/a",
            "title": "My Title",
            "date_field": "2023-07-03",
            "enum_field": "enum_val_2",
            "list_field": [
                {
                    "enum_in_object": "enum_in_obj_2"
                },
                {
                    "enum_in_object": "enum_in_obj_3"
                }
            ],
            "user_field": "788dcb76-9928-46fc-87ba-7043708f1bc0",
            "number_field": 1337.0,
            "object_field": {
                "list_in_object": [
                    "My String in List in Object",
                    "Other String in List in Object"
                ]
            },
            "boolean_field": true,
            "combobox_field": "Combobox Value 2",
            "markdown_field": "My Markdown"
        }
    }"""

    finding_with_predefined_fields = """
    {
        "id": "c8941493-c5e2-4a89-b82e-a3513f54c1b4",
        "created": "2023-08-22T09:22:38.896650Z",
        "updated": "2023-08-22T09:22:38.899219Z",
        "project": "4820bd5d-51f1-4dca-a4a4-78ba115b615c",
        "project_type": "c9b00138-8079-40d8-8030-1aecf3b66e11",
        "language": "en-US",
        "lock_info": {},
        "template": null,
        "assignee": {},
        "status": "in-progress",
        "order": 1,
        "data": {
            "cvss": "n/a",
            "title": "Test",
            "impact": "TODO: impact of finding",
            "summary": "TODO: High-level summary",
            "severity": null,
            "references": [],
            "description": "TODO: detailed technical description what this findings is about and how it can be exploited",
            "precondition": null,
            "retest_notes": null,
            "retest_status": null,
            "wstg_category": null,
            "recommendation": "TODO: how to fix the vulnerability",
            "owasp_top10_2021": null,
            "affected_components": [],
            "short_recommendation": "TODO: short recommendation"
        }
    }
"""
    incompatible_finding = """
    {
        "id": "c8941493-c5e2-4a89-b82e-a3513f54c1b4",
        "created": "2023-08-22T09:22:38.896650Z",
        "updated": "2023-08-22T09:22:38.899219Z",
        "project": "4820bd5d-51f1-4dca-a4a4-78ba115b615c",
        "project_type": "c9b00138-8079-40d8-8030-1aecf3b66e11",
        "language": "en-US",
        "lock_info": {},
        "template": null,
        "assignee": {},
        "status": "in-progress",
        "order": 1,
        "data": {
            "cvss": "n/a",
            "title": "Test",
            "impact": "TODO: impact of finding",
            "affected_components": [],
            "short_recommendation": "TODO: short recommendation",
            "not_predefined_field": "definitely not predefined."
        }
    }
"""

    def test_incompatible_finding_without_design(self):
        finding_raw = FindingRaw(json.loads(self.incompatible_finding))
        Finding(finding_raw, force_compatible=False)
        with pytest.raises(IncompatibleDesignException):
            Finding(finding_raw, force_compatible=True)
        with pytest.raises(IncompatibleDesignException):
            Finding(finding_raw)

    def test_finding_unwanted_casting(self):
        finding = Finding(json.loads(self.finding_with_predefined_fields))
        assert isinstance(finding, Finding)
        assert isinstance(finding.data, FindingData)
        assert isinstance(finding.data.title, FindingDataField)
        finding.to_dict()
        assert isinstance(finding, Finding)
        assert isinstance(finding.data, FindingData)
        assert isinstance(finding.data.title, FindingDataField)

    def test_finding_without_design_from_dict(self):
        finding = Finding(json.loads(self.finding_with_predefined_fields))
        assert finding.id == "c8941493-c5e2-4a89-b82e-a3513f54c1b4"
        assert finding.data.title.value == "Test"
        finding.data.title.value = "New"
        with pytest.raises(ValueError):
            finding.data.title.value = 12

        finding.data.cvss.value = "n/a"
        with pytest.raises(ValueError):
            finding.data.cvss.value = 12

        finding.data.impact.value = "new"
        with pytest.raises(ValueError):
            finding.data.impact.value = 12

        finding.data.summary.value = "new"
        with pytest.raises(ValueError):
            finding.data.summary.value = 12

        finding.data.severity.value = "info"
        with pytest.raises(ValueError):
            finding.data.severity.value = 12
        with pytest.raises(ValueError):
            finding.data.severity.value = "very_high"

        finding.data.references.value = ["new"]
        with pytest.raises(ValueError):
            finding.data.references.value = 12
        with pytest.raises(ValueError):
            finding.data.references.value = ["new", 12]
        with pytest.raises(ValueError):
            finding.data.references.value = "abc"

        finding.data.description.value = "new"
        with pytest.raises(ValueError):
            finding.data.description.value = 12

        finding.data.precondition.value = "new"
        with pytest.raises(ValueError):
            finding.data.precondition.value = 12

        finding.data.retest_notes.value = "new"
        with pytest.raises(ValueError):
            finding.data.retest_notes.value = 12

        finding.data.retest_status.value = "open"
        with pytest.raises(ValueError):
            finding.data.retest_status.value = 12
        with pytest.raises(ValueError):
            finding.data.retest_status.value = "invalid"

        finding.data.wstg_category.value = "INFO"
        with pytest.raises(ValueError):
            finding.data.wstg_category.value = 12
        with pytest.raises(ValueError):
            finding.data.wstg_category.value = "invalid"

        finding.data.recommendation.value = "new"
        with pytest.raises(ValueError):
            finding.data.recommendation.value = 12

        finding.data.owasp_top10_2021.value = "A01_2021"
        with pytest.raises(ValueError):
            finding.data.owasp_top10_2021.value = 12
        with pytest.raises(ValueError):
            finding.data.owasp_top10_2021.value = "invalid"

        finding.data.affected_components.value = ["new"]
        with pytest.raises(ValueError):
            finding.data.affected_components.value = 12
        with pytest.raises(ValueError):
            finding.data.affected_components.value = ["new", 12]
        with pytest.raises(ValueError):
            finding.data.affected_components.value = "abc"

        finding.data.short_recommendation.value = "new"
        with pytest.raises(ValueError):
            finding.data.short_recommendation.value = 12

    def test_finding(self):
        finding_raw = FindingRaw(json.loads(self.example_finding))
        project_design = ProjectDesign(
            json.loads(self.example_design_with_finding_fields_only)
        )
        finding = Finding(finding_raw, project_design=project_design)
        assert finding.id == "d3658ee5-2d43-40f6-9b97-1b98480afe78"
        assert finding.data.boolean_field.value == True
        assert finding.data.boolean_field.type == "boolean"
        assert finding.data.boolean_field.name == "boolean_field"

    def test_finding_data(self):
        json_example = json.loads(self.example_finding)
        finding_raw = FindingRaw(json_example)
        project_design = ProjectDesign(
            json.loads(self.example_design_with_finding_fields_only)
        )
        finding_data = FindingData(project_design.finding_fields, finding_raw.data)
        assert finding_data.boolean_field.name == "boolean_field"
        assert finding_data.boolean_field.type == "boolean"
        assert finding_data.boolean_field.value == True

        assert finding_data.combobox_field.name == "combobox_field"
        assert finding_data.combobox_field.type == "combobox"
        assert finding_data.combobox_field.value == "Combobox Value 2"

        assert finding_data.cvss.name == "cvss"
        assert finding_data.cvss.type == "cvss"
        assert finding_data.cvss.value == "n/a"

        assert finding_data.date_field.name == "date_field"
        assert finding_data.date_field.type == "date"
        assert finding_data.date_field.value == "2023-07-03"

        assert finding_data.enum_field.name == "enum_field"
        assert finding_data.enum_field.type == "enum"
        assert finding_data.enum_field.value == "enum_val_2"

        assert finding_data.list_field.name == "list_field"
        assert finding_data.list_field.type == "list"
        assert isinstance(finding_data.list_field.value, list)
        assert isinstance(finding_data.list_field.value[0], FindingDataField)
        assert finding_data.list_field.value[0].type == "object"
        assert isinstance(finding_data.list_field.value[0].value, dict)
        assert isinstance(
            finding_data.list_field.value[0].value["enum_in_object"], FindingDataField
        )
        assert finding_data.list_field.value[0].value["enum_in_object"].type == "enum"
        assert (
            finding_data.list_field.value[0].value["enum_in_object"].value
            == "enum_in_obj_2"
        )
        assert finding_data.object_field.name == "object_field"
        assert finding_data.object_field.type == "object"
        assert isinstance(finding_data.object_field.value, dict)
        assert isinstance(
            finding_data.object_field.value["list_in_object"], FindingDataField
        )
        assert finding_data.object_field.value["list_in_object"].type == "list"
        assert isinstance(finding_data.object_field.value["list_in_object"].value, list)
        assert isinstance(
            finding_data.object_field.value["list_in_object"].value[0], FindingDataField
        )
        assert (
            finding_data.object_field.value["list_in_object"].value[0].type == "string"
        )
        assert (
            finding_data.object_field.value["list_in_object"].value[0].value
            == "My String in List in Object"
        )

        assert finding_data.to_dict() == json_example["data"]
        assert not isinstance(finding_data, dict)

        # Try to set attributes
        finding_data.enum_field.value = "enum_val_1"
        assert finding_data.enum_field.value == "enum_val_1"
        with pytest.raises(ValueError):
            finding_data.enum_field.value = "invalid_value"

        finding_data.boolean_field.value = False
        assert finding_data.boolean_field.value == False
        with pytest.raises(ValueError):
            finding_data.boolean_field.value = "invalid_value"

        finding_data.combobox_field.value = "Combobox Value 1"
        assert finding_data.combobox_field.value == "Combobox Value 1"
        with pytest.raises(ValueError):
            finding_data.combobox_field.value = True

        finding_data.date_field.value = "2005-01-01"
        assert finding_data.date_field.value == "2005-01-01"
        with pytest.raises(ValueError):
            finding_data.date_field.value = "2002-01-32"

        new_list = deepcopy(finding_data.list_field.value)
        new_list[0].value["enum_in_object"].value = "enum_in_obj_1"
        finding_data.list_field.value = new_list
        assert finding_data.list_field.value == new_list
        with pytest.raises(ValueError):
            # ValueError due to invalid enum
            new_list[0].value["enum_in_object"].value = "invalid_value"
        with pytest.raises(ValueError):
            finding_data.list_field.value = "invalid_value"
        with pytest.raises(ValueError):
            finding_data.list_field.value = [new_list[0], "invalid_value"]
        with pytest.raises(ValueError):
            finding_data.list_field.value = [new_list[0], finding_data.combobox_field]

        new_object = deepcopy(finding_data.object_field.value)
        new_object["list_in_object"].value[0].value = "My new String in List in Object"
        finding_data.object_field.value = new_object
        assert finding_data.object_field.value == new_object
        with pytest.raises(ValueError):
            # ValueError due to invalid string
            new_object["list_in_object"].value[0].value = 1
        with pytest.raises(ValueError):
            finding_data.object_field.value = "invalid_value"
        with pytest.raises(ValueError):
            finding_data.object_field.value = [
                new_object["list_in_object"],
                "invalid_value",
            ]
        with pytest.raises(ValueError):
            finding_data.object_field.value = [
                new_object["list_in_object"],
                finding_data.combobox_field,
            ]

        with pytest.raises(ValueError):
            finding_data.number_field.value = "invalid_value"

        finding_data.user_field.value = "3cb580bd-131b-48f0-9e37-b405e2ab53b8"
        with pytest.raises(ValueError):
            finding_data.user_field.value = "invalid_value"

    def test_finding_raw_parsing(self):
        api_test_data = json.loads(self.example_finding)
        finding = FindingRaw(api_test_data)
        assert finding.id == "d3658ee5-2d43-40f6-9b97-1b98480afe78"
        assert finding.language == "en-US"
        assert finding.project_type == "2970149f-e11d-420a-8a5d-25b5fda14e33"
        assert isinstance(finding.assignee, dict)
        assert isinstance(finding.data, FindingDataRaw)
        assert finding.data.cvss == "n/a"
        assert finding.data.title == "My Title"
        assert finding.data.date_field == "2023-07-03"
        assert finding.data.enum_field == "enum_val_2"
        assert isinstance(finding.data.list_field, list)
        assert len(finding.data.list_field) == 2
        assert finding.data.list_field[0] == {"enum_in_object": "enum_in_obj_2"}
        assert finding.data.user_field == "788dcb76-9928-46fc-87ba-7043708f1bc0"
        assert finding.data.number_field == 1337.0
        assert isinstance(finding.data.number_field, float)
        assert isinstance(finding.data.object_field, dict)
        assert "list_in_object" in finding.data.object_field
        assert len(finding.data.object_field["list_in_object"]) == 2
        assert (
            finding.data.object_field["list_in_object"][0]
            == "My String in List in Object"
        )
        assert finding.data.boolean_field == True
        assert finding.data.combobox_field == "Combobox Value 2"
        assert finding.data.markdown_field == "My Markdown"



from typing import Any
from reptor.models.Base import BaseModel, ProjectFieldTypes
from reptor.settings import DEFAULT_PROJECT_DESIGN


import typing


class ProjectDesignField(BaseModel):
    """

    Attributes:
        name:
        type:
        label:
        origin:
        default:
        required:
        spellcheck:

        properties: "ProjectDesignField"
        choices:
        items:
        suggestions:
    """

    name: str = ""
    type: ProjectFieldTypes
    label: str = ""
    origin: str = ""
    default: str = ""
    required: bool = False
    spellcheck: bool = False
    # Use Any instead of "typing.List['ProjectDesignField'] = []" due to Python bug
    # See: https://bugs.python.org/issue44926
    properties: Any = None
    choices: typing.List[dict] = []
    items: dict = {}
    suggestions: typing.List[str] = []

    def _fill_from_api(self, data: typing.Dict):
        if data["type"] == ProjectFieldTypes.list.value:
            if isinstance(data["items"], dict):
                data["items"] = ProjectDesignField(data["items"])
        elif data["type"] == ProjectFieldTypes.object.value:
            properties = list()
            for name, field in data["properties"].items():
                field["name"] = name
                properties.append(ProjectDesignField(field))
            data["properties"] = properties

        attrs = typing.get_type_hints(self).keys()
        for key, value in data.items():
            if key in attrs:
                self.__setattr__(key, value)


class ProjectDesign(BaseModel):
    """
    Attributes:
        source:
        scope:
        name:
        language:
        report_fields:
        finding_fields:
    """

    source: str = ""
    scope: str = ""
    name: str = ""
    language: str = ""
    report_fields: typing.List[ProjectDesignField] = []
    finding_fields: typing.List[ProjectDesignField] = []

    def __init__(self, data: typing.Optional[typing.Dict] = None):
        if data is None:
            data = DEFAULT_PROJECT_DESIGN
        super().__init__(data)



import re
from typing import Union

from requests.exceptions import HTTPError
from reptor.models.Finding import Finding
from reptor.models.Section import Section

from reptor.models.Base import ProjectFieldTypes
from reptor.lib.plugins.Base import Base

try:
    import deepl
except ImportError:
    deepl = None


class Translate(Base):
    """ """

    meta = {
        "name": "Translate",
        "summary": "Translate Projects to other languages via Deepl",
    }

    PREDEFINED_SKIP_FIELDS = [
        "affected_components",
        "references",
    ]
    TRANSLATE_DATA_TYPES = [
        ProjectFieldTypes.string.value,
        ProjectFieldTypes.markdown.value,
    ]
    DEEPL_FROM = [
        "BG",
        "CS",
        "DA",
        "DE",
        "EL",
        "EN",
        "ES",
        "ET",
        "FI",
        "FR",
        "HU",
        "ID",
        "IT",
        "JA",
        "KO",
        "LT",
        "LV",
        "NB",
        "NL",
        "PL",
        "PT",
        "RO",
        "RU",
        "SK",
        "SL",
        "SV",
        "TR",
        "UK",
        "ZH",
    ]
    DEEPL_TO = [
        "BG",
        "CS",
        "DA",
        "DE",
        "EL",
        "EN",
        "EN-GB",
        "EN-US",
        "ES",
        "ET",
        "FI",
        "FR",
        "HU",
        "ID",
        "IT",
        "JA",
        "KO",
        "LT",
        "LV",
        "NB",
        "NL",
        "PL",
        "PT",
        "PT-BR",
        "PT-PT",
        "RO",
        "RU",
        "SK",
        "SL",
        "SV",
        "TR",
        "UK",
        "ZH",
    ]

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.from_lang = kwargs.get("from")
        self.to_lang = kwargs["to"]
        self.dry_run = kwargs.get("dry_run")
        self.chars_count_to_translate = 0
        try:
            self.skip_fields = kwargs.get("skip_fields", "").split(",") or getattr(
                self, "skip_fields"
            )
        except AttributeError:
            self.skip_fields = list()
        try:
            self.skip_fields.extend(self.PREDEFINED_SKIP_FIELDS)
        except TypeError:
            raise TypeError(f"skip_fields should be list.")
        if not hasattr(self, "deepl_api_token"):
            self.deepl_api_token = ""

        try:
            if not self.deepl_api_token:
                # TODO error msg might propose a conf command for interactive configuration
                raise AttributeError("No Deepl API token found.")
            if not deepl:
                raise ModuleNotFoundError(
                    'deepl library not found. Install plugin requirements with "pip3 install reptor[translate]'
                )
            self.deepl_translator = deepl.Translator(self.deepl_api_token)
        except (AttributeError, ModuleNotFoundError) as e:
            if not self.dry_run:
                raise e

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath)
        parser.add_argument(
            "-from",
            "--from",
            metavar="LANGUAGE_CODE",
            help="Language code of source language",
            choices=cls.DEEPL_FROM,
            action="store",
            default=None,
        )
        parser.add_argument(
            "-to",
            "--to",
            metavar="LANGUAGE_CODE",
            help="Language code of dest language",
            choices=cls.DEEPL_TO,
            action="store",
            default=None,
        )
        parser.add_argument(
            "-skip-fields",
            "--skip-fields",
            metavar="FIELDS",
            help="Report and Finding fields, comma-separated",
            action="store",
            default="",
        )

        # Currently supported: Deepl
        # parser.add_argument(
        #    "-translator",
        #    "--translator",
        #    help="Translator service to use",
        #    choices=["deepl"],
        #    default="deepl",
        # )
        parser.add_argument(
            "-dry-run",
            "--dry-run",
            help="Do not translate, count characters to be translated and checks Deepl quota",
            action="store_true",
        )

    def _translate(self, text: str) -> str:
        if not re.search("[a-zA-Z]", text):
            return text

        self.chars_count_to_translate += len(text)

        result = self.deepl_translator.translate_text(
            text,
            source_lang=self.from_lang,  # Can be None
            target_lang=self.to_lang,
            preserve_formatting=True,
        )
        return result.text

    def _translate_section(
        self, section: Union[Finding, Section]
    ) -> Union[Finding, Section]:
        for field in section.data:
            if field.type not in self.TRANSLATE_DATA_TYPES:
                continue
            if field.name in self.skip_fields:
                continue
            field.value = self._translate(field.value)
        return section

    def _dry_run_translate(self, text: str) -> str:
        self.chars_count_to_translate += len(text)
        return text

    def _duplicate_and_update_project(self, project_title: str) -> None:
        self.display(f"Duplicating project{' (dry run)' if self.dry_run else ''}.")
        if not self.dry_run:
            to_project_id = self.reptor.api.projects.duplicate().id
            self.display(
                f"Updating project metadata{' (dry run)' if self.dry_run else ''}."
            )
            # Switch project to update duplicated project instead of original
            self.reptor.api.switch_project(to_project_id)

            try:
                data = {"name": self._translate(project_title)}
                if sysreptor_language_code := self._get_sysreptor_language_code(
                    self.to_lang
                ):
                    data["language"] = sysreptor_language_code
                self.reptor.api.projects.update_project(data)
            except HTTPError as e:
                self.warning(f"Error updating project: {e.response.text}")
        else:
            self._translate(project_title)  # To count characters

    def _translate_project(self):
        if self.dry_run:
            self._translate = self._dry_run_translate

        project = self.reptor.api.projects.get_project()
        self._duplicate_and_update_project(project_title=project.name)

        self.display(f"Translating findings{' (dry run)' if self.dry_run else ''}.")
        sections = [
            Finding(f, self.reptor.api.project_designs.project_design)
            for f in self.reptor.api.projects.get_findings()
        ] + self.reptor.api.projects.get_sections()
        for section in sections:
            translated_section = self._translate_section(section)
            translated_section_data = translated_section.data.to_dict()
            if not self.dry_run:
                if translated_section.__class__ == Finding:
                    self.reptor.api.projects.update_finding(
                        translated_section.id, {"data": translated_section_data}
                    )
                elif translated_section.__class__ == Section:
                    self.reptor.api.projects.update_section(
                        translated_section.id, {"data": translated_section_data}
                    )

        self.display(
            f"Translated {self.chars_count_to_translate} characters{' (dry run)' if self.dry_run else ''}."
        )
        self._log_deepl_usage()
        self.success(f"Project translated{' (dry run)' if self.dry_run else ''}.")
        self.display(
            "We recommend to check quality of the translation, or to add a note that the report was "
            "translated by a machine."
        )

    def _log_deepl_usage(self):
        try:
            usage = self.deepl_translator.get_usage()
            if usage.any_limit_reached:
                self.warning("Deepl transaction limit reached.")
            if usage.character.valid:
                self.display(
                    f"Deepl usage: {usage.character.count} of {usage.character.limit} characters"
                )
        except AttributeError:
            pass

    def _get_sysreptor_language_code(self, language_code) -> str:
        enabled_language_codes = self.reptor.api.projects.get_enabled_language_codes()
        matched_lcs = [
            enabled_lc
            for enabled_lc in enabled_language_codes
            if enabled_lc.lower().startswith(language_code[:2].lower())
        ]
        if not matched_lcs:
            return ""
        elif len(matched_lcs) == 1:
            return matched_lcs[0]
        else:
            for matched_lc in matched_lcs:
                if matched_lc.lower() == language_code.lower():
                    return matched_lc
            else:
                return matched_lcs[0]

    def run(self):
        self._translate_project()


loader = Translate



import json
from copy import deepcopy

import pytest

from reptor.api.manager import APIManager
from reptor.models.Finding import FindingRaw
from reptor.models.ProjectDesign import ProjectDesign
from reptor.models.Finding import Finding
from reptor.lib.reptor import Reptor

from ..Translate import Translate


class TestTranslate:
    example_finding = """
    {
        "id": "d3658ee5-2d43-40f6-9b97-1b98480afe78",
        "created": "2023-07-16T20:07:36.213385Z",
        "updated": "2023-07-16T20:08:59.749591Z",
        "project": "a4b4b630-fc78-452d-b348-d362b69c2449",
        "project_type": "2970149f-e11d-420a-8a5d-25b5fda14e33",
        "language": "en-US",
        "lock_info": null,
        "template": null,
        "assignee": {
            "id": "788dcb76-9928-46fc-87ba-7043708f1bc0",
            "username": "demo-fTaIO4fj",
            "name": "",
            "title_before": null,
            "first_name": "",
            "middle_name": null,
            "last_name": "",
            "title_after": null,
            "is_active": true
        },
        "status": "in-progress",
        "data": {
            "cvss": "n/a",
            "title": "My Title",
            "date_field": "2023-07-03",
            "enum_field": "enum_val_2",
            "list_field": [
                {
                    "enum_in_object": "enum_in_obj_2"
                },
                {
                    "enum_in_object": "enum_in_obj_3"
                }
            ],
            "user_field": "788dcb76-9928-46fc-87ba-7043708f1bc0",
            "number_field": 1337.0,
            "object_field": {
                "list_in_object": [
                    "My String in List in Object",
                    "Other String in List in Object"
                ]
            },
            "boolean_field": true,
            "combobox_field": "Combobox Value 2",
            "markdown_field": "My Markdown"
        }
    }"""
    example_design_with_finding_fields_only = """
    {"finding_fields":{"cvss":{"type":"cvss","label":"CVSS","origin":"core","default":"n/a","required":true},"title":{"type":"string","label":"Title","origin":"core","default":"TODO: Finding Title","required":true,"spellcheck":true},"date_field":{"type":"date","label":"Date Field","origin":"custom","default":null,"required":true},"enum_field":{"type":"enum","label":"Enum Field","origin":"custom","choices":[{"label":"Enum Value 1","value":"enum_val_1"},{"label":"Enum Value 2","value":"enum_val_2"},{"label":"Enum Value 3","value":"enum_val_"}],"default":null,"required":true},"list_field":{"type":"list","items":{"type":"object","label":"","origin":"custom","properties":{"enum_in_object":{"type":"enum","label":"Enum in Object","origin":"custom","choices":[{"label":"Enum in Obj 1","value":"enum_in_obj_1"},{"label":"Enum in Obj 2","value":"enum_in_obj_2"},{"label":"Enum in Obj 3","value":"enum_in_obj_3"}],"default":null,"required":true}}},"label":"List Field","origin":"custom","required":true},"user_field":{"type":"user","label":"User Field","origin":"custom","required":true},"number_field":{"type":"number","label":"Number Field","origin":"custom","default":null,"required":true},"object_field":{"type":"object","label":"Object Field","origin":"custom","properties":{"list_in_object":{"type":"list","items":{"type":"string","label":"","origin":"custom","default":null,"required":true,"spellcheck":false},"label":"List in Object","origin":"custom","required":true}}},"boolean_field":{"type":"boolean","label":"Boolean Field","origin":"custom","default":null},"combobox_field":{"type":"combobox","label":"Combobox Field","origin":"custom","default":null,"required":true,"suggestions":["Combobox Value 1","Combobox Value 2","Combobox Value 3"]},"markdown_field":{"type":"markdown","label":"Markdown Field","origin":"custom","default":null,"required":true}}}"""

    @pytest.fixture(autouse=True)
    def setUp(self):
        class Translator:
            def translate_text(self, text, **kwargs):
                class Result:
                    def __init__(self, text):
                        self.text = text

                return Result(f"Translated: {text}")

        reptor = Reptor()
        reptor._config._raw_config[
            "project_id"
        ] = "8a6ebd7b-637f-4f38-bfdd-3e8e9a24f64e"
        reptor._api = APIManager(reptor=reptor)
        self.translate = Translate(reptor=reptor, to="EN", dry_run=True)
        self.translate.deepl_translator = Translator()

        finding_raw = FindingRaw(json.loads(self.example_finding))
        project_design = ProjectDesign(
            json.loads(self.example_design_with_finding_fields_only)
        )
        self.finding = Finding(finding_raw, project_design=project_design)

    def test_translate(self):
        assert self.translate.chars_count_to_translate == 0
        assert self.translate._dry_run_translate("12345") == "12345"
        assert self.translate.chars_count_to_translate == 5

        text = "Hello World"
        assert self.translate._translate(text) == f"Translated: {text}"
        assert self.translate.chars_count_to_translate == 5 + len(text)
        assert self.translate._translate("123") == f"123"
        assert self.translate.chars_count_to_translate == 5 + len(text)

        self.translate.skip_fields = ["title"]
        translated_finding = self.translate._translate_section(deepcopy(self.finding))

        # Fields that should be translated
        assert (
            translated_finding.data.markdown_field.value
            == f"Translated: {self.finding.data.markdown_field.value}"
        )
        assert (
            translated_finding.data.object_field.value["list_in_object"].value[0].value
            == f"Translated: {self.finding.data.object_field.value['list_in_object'].value[0].value}"
        )

        # Fields not translated due to skip_fields
        assert translated_finding.data.title.value == self.finding.data.title.value

        # Fields that should not be translated
        assert (
            translated_finding.data.combobox_field.value
            == self.finding.data.combobox_field.value
        )
        assert translated_finding.data.cvss.value == self.finding.data.cvss.value
        assert (
            translated_finding.data.enum_field.value
            == self.finding.data.enum_field.value
        )
        assert (
            translated_finding.data.list_field.value[0].value["enum_in_object"].value
            == self.finding.data.list_field.value[0].value["enum_in_object"].value
        )

    def test_language_code_mapping(self):
        class Reptor:
            class Api:
                class Projects:
                    def get_enabled_language_codes(self) -> list:
                        return ["en-US", "de-DE", "es-ES", "fr-FR", "de-XX"]

                projects = Projects()

            api = Api()

        self.translate.reptor = Reptor()
        assert self.translate._get_sysreptor_language_code("EN") == "en-US"
        assert self.translate._get_sysreptor_language_code("EN-GB") == "en-US"
        assert self.translate._get_sysreptor_language_code("EN-US") == "en-US"

        assert self.translate._get_sysreptor_language_code("DE") == "de-DE"
        assert self.translate._get_sysreptor_language_code("DE-xX") == "de-XX"
        assert self.translate._get_sysreptor_language_code("ES") == "es-ES"
        assert self.translate._get_sysreptor_language_code("FR") == "fr-FR"

        assert self.translate._get_sysreptor_language_code("invalid") == ""



from rich.markdown import Markdown


def convert_markdown_to_console(markdown_in) -> Markdown:
    """Converts text markdown to rich.Console parsed markdown

    Args:
        markdown_in (str): raw markdown text

    Returns:
        Markdown: parsed Markdown from rich
    """
    return Markdown(markdown_in)



import typing

from reptor.lib.plugins.ModelBase import ModelBase


class Instance(ModelBase):
    uri: str
    method: str
    param: str
    attack: str
    evidence: str
    otherinfo: str
    requestheader: str
    requestbody: str
    responseheader: str
    # responsebody: str # Careful with this!

    def parse(self, data):
        self.uri = data.find("uri").text
        self.method = data.find("method").text
        self.param = data.find("param").text
        self.attack = data.find("attack").text
        self.evidence = data.find("evidence").text
        self.otherinfo = data.find("otherinfo").text
        if data.find("requestheader"):
            self.requestheader = data.find("requestheader").text
        if data.find("requestbody"):
            self.requestbody = data.find("requestbody").text
        if data.find("responseheader"):
            self.responseheader = data.find("responseheader").text


class Alert(ModelBase):
    pluginid: str
    alertRef: str
    name: str
    riskcode: int
    confidence: int
    riskdesc: str
    confidencedesc: str
    desc: str
    count: int
    solution: str
    otherinfo: str
    reference: str
    cweid: int
    wascid: int
    sourceid: int

    instances: typing.List[Instance] = list()

    def parse(self, data):
        self.pluginid = data.find("pluginid").text
        self.alertRef = data.find("alertRef").text
        self.name = data.find("name").text
        self.riskcode = data.find("riskcode").text
        self.confidence = data.find("confidence").text
        self.riskdesc = data.find("riskdesc").text
        self.confidencedesc = data.find("confidencedesc").text
        self.desc = data.find("desc").text
        self.count = data.find("count").text
        self.solution = data.find("solution").text
        self.reference = data.find("reference").text
        self.cweid = data.find("cweid").text
        self.wascid = data.find("wascid").text
        self.sourceid = data.find("sourceid").text

    @property
    def references_as_list_items(self):
        return self.reference.splitlines()


class Site(ModelBase):
    name: str
    host: str
    port: str
    ssl: bool

    alerts: typing.List[Alert] = list()

    def parse(self, data):
        self.name = data.get("name")
        self.host = data.get("host")
        self.port = data.get("port")
        self.ssl = data.get("ssl")



import json

import pytest

from reptor.lib.conf import Config
from reptor.lib.reptor import Reptor

from .. import GhostWriter


class TestGhostwriter:
    @pytest.fixture(autouse=True)
    def setUp(self):
        self.reptor = Reptor()
        self.reptor._config = Config()
        self.reptor._config._raw_config = {}

    def test_init(self):
        self.reptor.get_config()
        with pytest.raises(ValueError):
            # Should raise because of missing URL
            GhostWriter.GhostWriter(reptor=self.reptor)

        with pytest.raises(ValueError):
            # Should raise because of missing API key
            GhostWriter.GhostWriter(reptor=self.reptor, url="test")

        GhostWriter.GhostWriter.apikey = "test"
        GhostWriter.GhostWriter(reptor=self.reptor, url="test")

    def test_get_findings(self):
        finding_data = json.loads(
            '{"finding": [{"cvss_vector": "CVSS:3.0/AV:N/AC:L/PR:L/UI:R/S:U/C:L/I:H/A:H", "description": "<p>This finding was part of the Ghostwriter installation.</p>", "impact": "<p>There is a significant risk of ghostwriter being used.</p>", "mitigation": "<p>We recommend to migrate to SysReptor.</p>", "references": "<ul>\\r\\n<li>Ghostwriter docs: https://www.ghostwriter.wiki/</li>\\r\\n<li>SysReptor docs: https://docs.sysreptor.com/</li>\\r\\n</ul>\\r\\n<p>\\u00a0</p>", "title": "High risk of using Ghostwriter"}]}'
        )

        class Client:
            def __init__(*args, **kwargs):
                pass

            def execute(self, *args, **kwargs):
                return finding_data

        # Test x-hasura-admin-secret
        def AIOHTTPTransport(url, headers):
            assert url == "http://localhost:8080/v1/graphql"
            assert headers == {
                "Content-Type": "application/json",
                "x-hasura-admin-secret": "123456789",
            }

        GhostWriter.AIOHTTPTransport = AIOHTTPTransport
        GhostWriter.Client = Client
        GhostWriter.GhostWriter.apikey = "123456789"
        ghostwriter = GhostWriter.GhostWriter(
            reptor=self.reptor, url="http://localhost:8080"
        )
        assert finding_data["finding"] == ghostwriter._get_ghostwriter_findings()

        # Test Authorization Bearer
        jwt_token = "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJPbmxpbmUgSldUIEJ1aWxkZXIiLCJpYXQiOjE2OTE2NTkzNjUsImV4cCI6MTY5MTY1OTM3NiwiYXVkIjoid3d3LmV4YW1wbGUuY29tIiwic3ViIjoianJvY2tldEBleGFtcGxlLmNvbSIsIkdpdmVuTmFtZSI6IkpvaG5ueSIsIlN1cm5hbWUiOiJSb2NrZXQiLCJFbWFpbCI6Impyb2NrZXRAZXhhbXBsZS5jb20iLCJSb2xlIjpbIk1hbmFnZXIiLCJQcm9qZWN0IEFkbWluaXN0cmF0b3IiXX0.xqxBhewa86-Hy2ZiiS_obrjyXZ8vRmqdnDFaFGvXjgU"

        def AIOHTTPTransport(url, headers):
            assert url == "http://localhost:8080/v1/graphql"
            assert headers == {
                "Content-Type": "application/json",
                "Authorization": f"Bearer {jwt_token}",
            }

        GhostWriter.AIOHTTPTransport = AIOHTTPTransport
        GhostWriter.Client = Client
        GhostWriter.GhostWriter.apikey = jwt_token
        ghostwriter = GhostWriter.GhostWriter(
            reptor=self.reptor, url="http://localhost:8080"
        )
        assert finding_data["finding"] == ghostwriter._get_ghostwriter_findings()

    @pytest.mark.parametrize(
        "text,converted",
        [
            ("123\n456", ["123", "456"]),
            ("<ul>\n<li>abc</li>\n<li>cde</li>\n</ul>", ["abc", "cde"]),
        ],
    )
    def test_convert_references(self, text, converted):
        GhostWriter.GhostWriter.apikey = "123456789"
        g = GhostWriter.GhostWriter(reptor=self.reptor, url="http://localhost:8080")
        assert g.convert_references(text) == converted

    @pytest.mark.parametrize(
        "text,converted",
        [
            ("", ""),
            ("ABC", "\n\n**Host Detection Techniques**\n\nABC"),
        ],
    )
    def test_convert_hostDetectionTechniques(self, text, converted):
        GhostWriter.GhostWriter.apikey = "123456789"
        g = GhostWriter.GhostWriter(reptor=self.reptor, url="http://localhost:8080")
        assert g.convert_hostDetectionTechniques(text) == converted

    @pytest.mark.parametrize(
        "text,converted",
        [
            ("", ""),
            ("ABC", "\n\n**Network Detection Techniques**\n\nABC"),
        ],
    )
    def test_convert_networkDetectionTechniques(self, text, converted):
        GhostWriter.GhostWriter.apikey = "123456789"
        g = GhostWriter.GhostWriter(reptor=self.reptor, url="http://localhost:8080")
        assert g.convert_networkDetectionTechniques(text) == converted

    @pytest.mark.parametrize(
        "text,converted",
        [
            ("", ""),
            ("ABC", "TODO: ABC\n\n"),
        ],
    )
    def test_convert_findingGuidance(self, text, converted):
        GhostWriter.GhostWriter.apikey = "123456789"
        g = GhostWriter.GhostWriter(reptor=self.reptor, url="http://localhost:8080")
        assert g.convert_findingGuidance(text) == converted



from django import template

register = template.Library()

@register.tag
def noemptylines(parser, token):
    nodelist = parser.parse(("endnoemptylines",))
    parser.delete_first_token()
    return NoEmptyLinesNode(nodelist)


class NoEmptyLinesNode(template.Node):
    def __init__(self, nodelist):
        self.nodelist = nodelist

    def render(self, context):
        output = self.nodelist.render(context)
        rendered = list()
        for line in output.splitlines():
            if line.strip():
                rendered.append(line)
        return "\n".join(rendered)


import typing
from posixpath import join as urljoin

from reptor.api.APIClient import APIClient
from reptor.models.FindingTemplate import FindingTemplate


class TemplatesAPI(APIClient):
    def __init__(self, **kwargs) -> None:
        super().__init__(**kwargs)

        self.base_endpoint = urljoin(
            self.reptor.get_config().get_server(), f"api/v1/findingtemplates/"
        )
        self.object_endpoint = urljoin(f"api/v1/findingtemplates/{self.project_id}")

    def get_templates(self) -> typing.List[FindingTemplate]:
        """Gets list of Templates"""
        response = self.get(self.base_endpoint)
        return_data = list()
        for item in response.json()["results"]:
            return_data.append(FindingTemplate(item))
        return return_data

    def get_template(self, template_id: str) -> FindingTemplate:
        """Gets a single Template by ID"""
        response = self.get(urljoin(self.base_endpoint, template_id))
        return FindingTemplate(response.json())

    def search(self, search_term) -> typing.List[FindingTemplate]:
        """Searches through the templates"""

        response = self.get(urljoin(self.base_endpoint, f"?search={search_term}"))
        return_data = list()
        for item in response.json()["results"]:
            return_data.append(FindingTemplate(item))
        return return_data

    def upload_new_template(
        self, template: FindingTemplate
    ) -> typing.Optional[FindingTemplate]:
        """Uploads a new Finding Template to API

        Args:
            template (FindingTemplate): Model Data to upload

        Returns:
            FindingTemplate: Updated Model with ID etc.
        """
        res = self.post(
            self.base_endpoint,
            data=template.to_dict(),
        )
        return FindingTemplate(res.json())



from reptor.lib.plugins.ToolBase import ToolBase
from reptor.plugins.tools.Nmap.models import Service


class Nmap(ToolBase):
    """
    Author: Syslifters
    Version: 1.0
    Website: https://github.com/Syslifters/reptor
    License: MIT
    Tags: network,scanning,infrastructure

    Short Help:
    Formats nmap output

    Description:
    target=127.0.0.1
    sudo nmap -Pn -n -sV -oG - -p 1-65535 $target

    sudo -v  # Elevate privileges for non-interactive
    sudo -n nmap -Pn -n -sV -oG - -p 80,440 $target | tee nmap_result.txt | reptor nmap

    # Format and upload
    cat nmap_result.txt | reptor nmap -c upload
    """

    meta = {
        "name": "Nmap",
        "summary": "format nmap output",
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.notename = kwargs.get("notename", "Nmap Scan")
        self.note_icon = "👁️‍🗨️"
        if self.input_format == "raw":
            self.input_format = "xml"

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath)
        input_format_group = cls.get_input_format_group(parser)
        input_format_group.add_argument(
            "-oX",
            help="nmap XML output format, same as --xml (recommended)",
            action="store_const",
            dest="format",
            const="xml",
        )
        input_format_group.add_argument(
            "-oG",
            "--grepable",
            help="nmap Grepable output format",
            action="store_const",
            dest="format",
            const="grepable",
        )

        parser.add_argument(
            "-multi-notes",
            "--multi-notes",
            help="Uploads multiple notes (one per IP) instead of one note with all IPs",
            action="store_true",
        )

    def parse_grepable(self):
        self.parsed_input = list()
        for line in self.raw_input.splitlines():
            if line.startswith("#") or "Ports:" not in line:
                continue
            ip, ports = line.split("Ports:")
            ip = ip.split(" ")[1]

            ports = ports.split(",")
            for port in ports:
                port, status, protocol, _, service, _, version, _ = port.strip().split(
                    "/"
                )
                if status == "open":
                    s = Service()
                    s.parse(
                        {
                            "ip": ip,
                            "port": port,
                            "protocol": protocol,
                            "service": service.replace("|", "/"),
                            "version": version.replace("|", "/"),
                        }
                    )
                    self.parsed_input.append(s)

    def parse_xml(self):
        super().parse_xml()
        nmap_data = self.parsed_input
        self.parsed_input = list()
        hosts = nmap_data.get("nmaprun", {}).get("host", [])
        if not isinstance(hosts, list):
            hosts = [hosts]
        for host in hosts:
            ip = host.get("address", {}).get("@addr")
            ports = host.get("ports", {}).get("port", [])
            if not isinstance(ports, list):
                ports = [ports]
            for port in ports:
                if port.get("state", {}).get("@state") == "open":
                    s = Service()
                    s.parse(
                        {
                            "ip": ip,
                            "hostname": (host.get("hostnames") or {})
                            .get("hostname", {})
                            .get("@name"),
                            "port": port.get("@portid"),
                            "protocol": port.get("@protocol"),
                            "service": port.get("service", {}).get("@name"),
                            "version": port.get("service", {}).get("@product"),
                        }
                    )
                    self.parsed_input.append(s)

    def parse(self):
        super().parse()
        if self.input_format == "grepable":
            self.parse_grepable()

    def preprocess_for_template(self):
        data = dict()
        if not self.multi_notes:
            data["data"] = self.parsed_input
            data["show_hostname"] = any([s.hostname for s in self.parsed_input])
        else:
            # Group data per IP
            for d in self.parsed_input:
                # Key of the dict (d.ip) will be title of the note
                data.setdefault(d.ip, list()).append(d)
            # Show hostname or not
            for ip, port_data in data.items():
                data[ip] = {
                    "data": port_data,
                    "show_hostname": any([s.hostname for s in port_data]),
                }

        return data


loader = Nmap



import contextlib
import typing
from datetime import datetime
from os.path import basename
from posixpath import join as urljoin

from requests import HTTPError

from reptor.api.APIClient import APIClient
from reptor.lib.exceptions import LockedException
from reptor.models.Note import Note
from reptor.lib.errors import MissingArgumentError
from reptor.utils.file_operations import guess_filetype


class NotesAPI(APIClient):
    """Interacts with Notes Endpoints

    Args:
        APIClient (_type_): _description_
    """

    def __init__(self, **kwargs) -> None:
        super().__init__(require_project_id=False, **kwargs)

        if self.personal_note:
            self.base_endpoint = urljoin(
                self.reptor.get_config().get_server(),
                f"api/v1/pentestusers/self/notes/",
            )
        elif self.project_id:
            self.base_endpoint = urljoin(
                self.reptor.get_config().get_server(),
                f"api/v1/pentestprojects/{self.project_id}/notes/",
            )
        else:
            raise MissingArgumentError(
                "Either specify a project ID (-p|--project_id) or use --personal-note"
            )

    @property
    def personal_note(self):
        return self.reptor.get_config().get_cli_overwrite().get("personal_note")

    def get_notes(self) -> typing.List[Note]:
        """Gets list of notes"""
        self.debug("Getting Notes List")
        response = self.get(self.base_endpoint)
        notes = list()
        for note_data in response.json():
            notes.append(Note(note_data))
        return notes

    def create_note(
        self, title="CLI Note", parent_id: str = "", order=None, checked=None, icon=None
    ) -> Note:
        self.debug("Creating Note")
        note = self.post(
            self.base_endpoint,
            {
                "order": order,
                "parent": parent_id or None,
                "checked": checked,
                "title": title,
            },
        ).json()
        self.debug(f"We created note with {note}")
        if icon:
            self.set_icon(note.get("id"), icon)
        return Note(note)

    def set_icon(self, notes_id: str, icon: str):
        url = urljoin(self.base_endpoint, f"{notes_id}/")
        self.put(url, {"icon_emoji": icon})

    def write_note(
        self,
        content: str,
        notename: str = "",
        parent_notename: str = "",
        icon: str = "",
        no_timestamp: bool = False,
        force_unlock: bool = False,
    ):
        note = self.get_note_by_title(
            notename, parent_notename=parent_notename, icon=icon
        )
        self.debug(f"Working with note: {note.id}")

        with self._auto_lock_note(
            note.id
        ) if not force_unlock else contextlib.nullcontext():
            note_text = note.text + "\n\n"
            if not no_timestamp:
                note_text += f"[{datetime.now().strftime('%Y-%m-%d %H:%M:%S')}]"
                if "\n" in content:
                    if not content.startswith("\n"):
                        note_text += "\n"
                else:
                    note_text += ": "

            note_text += content
            self.debug(f"We are sending data with a lenght of: {len(note_text)}")
            url = urljoin(self.base_endpoint, note.id, "")
            r = self.put(url, {"text": note_text})

            try:
                r.raise_for_status()
                self.info(f'Note written to "{notename}".')
            except HTTPError as e:
                raise HTTPError(
                    f'{str(e)} Are you uploading binary content to note? (Try "file" subcommand)'
                ) from e

    def get_note_by_title(self, title, parent_notename=None, icon=None) -> Note:
        parent_id = None
        if parent_notename:
            note = self.get_note_by_title(parent_notename, icon=icon)
            parent_id = note.id
        notes_list = self.get_notes()

        for note in reversed(notes_list):
            if note.title == title and note.parent == parent_id:
                break
        else:
            # Note does not exist. Create.
            if title == "Uploads":
                icon = "📤"
            note = self.create_note(title=title, parent_id=parent_id, icon=icon)

        return note

    def upload_file(
        self,
        files=None,
        filename=None,
        caption=None,
        notename=None,
        parent_notename=None,
        icon: str = "",
        no_timestamp=False,
        force_unlock=False,
    ):
        if not files:
            return

        for file in files:
            if file.name == "<stdin>":
                self.info("Reading from stdin...")
            else:
                filename = basename(file.name)
            content = file.buffer.read()
            if not filename:
                filetype = guess_filetype(content) or "dat"
                filename = f"data.{filetype}"

            if not content:
                self.warning(f"{file.name} is empty. Will not upload.")
                continue

            # Lock during upload to prevent unnecessary uploads and for endpoint setup
            note = self.get_note_by_title(notename, parent_notename=parent_notename)
            if self.personal_note:
                url = urljoin(self.base_endpoint, "upload/")
            else:
                url = urljoin(self.base_endpoint.rsplit("/", 2)[0], "upload/")
            with self._auto_lock_note(
                note.id
            ) if not force_unlock else contextlib.nullcontext():
                # TODO this might be streamed
                files = {"file": (filename, content)}
                response_json = self.post(url, files=files, json_content=False).json()
                is_image = (
                    True if response_json.get("resource_type") == "image" else False
                )
                if is_image:
                    file_path = f"/images/name/{response_json['name']}"
                    note_content = f"\n![{caption or filename}]({file_path})"
                else:
                    file_path = f"/files/name/{response_json['name']}"
                    note_content = f"\n[{caption or filename}]({file_path})"

                self.write_note(
                    notename=notename,
                    content=note_content,
                    parent_notename=parent_notename,
                    icon=icon,
                    no_timestamp=no_timestamp,
                    force_unlock=True,
                )

    def _lock_note(self, note_id: str):
        url = urljoin(self.base_endpoint, note_id, "lock/")
        return self.post(url)

    def _unlock_note(self, note_id: str):
        url = urljoin(self.base_endpoint, note_id, "unlock/")
        return self.post(url)

    @contextlib.contextmanager
    def _auto_lock_note(self, note_id: str):
        try:
            r = self._lock_note(note_id)
        except HTTPError as e:
            if e.response.status_code == 403:
                raise LockedException("Cannot force unlock. Locked by other user.")
            else:
                raise e
        if r.status_code == 200:
            raise LockedException(
                "The section you want to write to is locked. "
                "(Unlock or force: --force-unlock)"
            )

        yield

        self._unlock_note(note_id)



from reptor.lib.plugins.DocParser import DocParser


class TestDocParser:
    example_docs = {
        "author": "Willy Wonka",
        "version": "1.5",
        "website": "https://github.com/Syslifters/reptor",
        "license": "MIT",
        "summary": "Parses OWASPZap XML and JSON reports",
    }

    def test_perfect_docs(self):
        module_docs = DocParser.parse(self.example_docs)

        assert module_docs.author == "Willy Wonka"
        assert module_docs.version == "1.5"
        assert module_docs.website == "https://github.com/Syslifters/reptor"
        assert module_docs.license == "MIT"
        assert module_docs.summary == "Parses OWASPZap XML and JSON reports"



from reptor.lib.plugins.ModelBase import ModelBase


class Service(ModelBase):
    ip: str
    port: int
    protocol: str
    service: str
    version: str

    def parse(self, data):
        self.ip = data.get("ip", "")
        self.hostname = data.get("hostname", "")
        self.port = data.get("port")
        self.protocol = data.get("protocol")
        self.service = data.get("service", "")
        self.version = data.get("version", "")



import datetime
import enum
import typing
from copy import deepcopy
from dataclasses import dataclass


@dataclass
class BaseModel:
    """
    Base Model
    """

    id: str = ""
    created: datetime.datetime = datetime.datetime.now()
    updated: datetime.datetime = datetime.datetime.now()

    def __init__(self, data: typing.Optional[typing.Dict] = None):
        if data:
            self._fill_from_api(data)

    def _get_combined_class_type_hints(self) -> dict:
        type_hints = list()
        type_hints_from_class = self.__class__
        while type_hints_from_class.__base__:
            type_hints.append(typing.get_type_hints(type_hints_from_class))
            if type_hints_from_class == BaseModel:
                break
            type_hints_from_class = type_hints_from_class.__base__

        combined_class_type_hints = dict()
        for type_hint in reversed(type_hints):
            combined_class_type_hints.update(type_hint)
        return combined_class_type_hints

    def _fill_from_api(self, data: typing.Dict):
        """Fills Model from reptor.api return JSON data

        Args:
            data (str): API Return Data
        """
        # Import here to prevent circular imports
        from reptor.models.Finding import FindingDataRaw
        from reptor.models.FindingTemplate import (
            FindingTemplate,
            FindingTemplateTranslation,
        )
        from reptor.models.Note import Note
        from reptor.models.Project import Project
        from reptor.models.ProjectDesign import ProjectDesign, ProjectDesignField
        from reptor.models.Section import SectionDataRaw
        from reptor.models.User import User
        from reptor.models.Finding import Finding
        from reptor.models.Section import Section

        combined_class_type_hints = self._get_combined_class_type_hints()

        for attr in combined_class_type_hints.items():
            if attr[0] in data:
                # Check if one of our models, then proceed recursively
                try:
                    model_class = attr[1].__args__[0]
                    is_list = True
                except AttributeError:
                    model_class = attr[1]
                    is_list = False

                if model_class in [
                    User,
                    Finding,
                    Section,
                    FindingTemplate,
                    FindingDataRaw,
                    SectionDataRaw,
                    Note,
                    Project,
                    ProjectDesign,
                    FindingTemplateTranslation,
                ]:
                    if is_list:
                        item_list = list()
                        for item in data[attr[0]]:
                            item_list.append(model_class(item))
                        self.__setattr__(attr[0], item_list)
                    else:
                        self.__setattr__(attr[0], model_class(data[attr[0]]))
                elif model_class == ProjectDesignField:
                    self.__setattr__(attr[0], list())
                    for k, v in data[attr[0]].items():
                        v["name"] = k
                        self.__getattribute__(attr[0]).append(model_class(v))
                else:
                    # Fill each attribute
                    self.__setattr__(attr[0], data[attr[0]])

    def to_dict(self):
        dict_values = deepcopy(vars(self))
        for k, v in dict_values.items():
            if isinstance(v, datetime.datetime):
                dict_values[k] = v.isoformat()
            elif isinstance(v, list):
                try:
                    dict_values[k] = [i.to_dict() for i in v]
                except AttributeError:
                    dict_values[k] = v
            else:
                try:
                    dict_values[k] = v.to_dict()  # type: ignore
                except AttributeError:
                    pass
        return dict_values


class ProjectFieldTypes(enum.Enum):
    cvss = "cvss"
    string = "string"
    markdown = "markdown"
    list = "list"
    object = "object"
    enum = "enum"
    user = "user"
    combobox = "combobox"
    date = "date"
    number = "number"
    boolean = "boolean"


class FindingTemplateSources(enum.Enum):
    CREATED = "created"
    IMPORTED = "imported"
    IMPORTED_DEPENDENCY = "imported_dependency"
    CUSTOMIZED = "customized"
    SNAPSHOT = "snapshot"



import datetime
import typing
from copy import deepcopy
from typing import Any
from uuid import UUID

from reptor.lib.exceptions import IncompatibleDesignException
from reptor.models.Base import BaseModel, ProjectFieldTypes
from reptor.models.ProjectDesign import ProjectDesign, ProjectDesignField


class SectionDataRaw(BaseModel):
    def _fill_from_api(self, data: typing.Dict):
        """Fills Model from reptor.api return JSON data
        For FindingDataRaw, undefined keys should also be set.

        Args:
            data (str): API Return Data
        """
        for key, value in data.items():
            # We don't care about recursion here
            self.__setattr__(key, value)


class SectionDataField(ProjectDesignField):
    """
    Section data holds values only and does not contain type definitions.
    Most data types cannot be differentiated (like strings and enums).

    This model joins finding data values from an acutal report with project
    design field definitions.
    """

    value: typing.Union[
        str,  # cvss, string, markdown, enum, user, combobox, date
        typing.List,  # list
        bool,  # boolean
        float,  # number
        Any,  # "SectionDataField" for object
    ]

    def __init__(
        self,
        design_field: ProjectDesignField,
        value: typing.Union[
            str,
            typing.List,
            bool,
            float,
            Any,
        ],
    ):
        # Set attributes from ProjectDesignField
        project_design_type_hints = typing.get_type_hints(ProjectDesignField)
        for attr in project_design_type_hints.items():
            self.__setattr__(attr[0], design_field.__getattribute__(attr[0]))

        if self.type == ProjectFieldTypes.object.value:
            property_value = dict()
            for property in self.properties:
                # property is of type ProjectDesignField
                try:
                    property_value[property.name] = self.__class__(
                        property, value[property.name]
                    )

                except KeyError:
                    raise KeyError(
                        f"Object name '{property.name}' not found. Did you mix"
                        f"mismatched project design with project data?"
                    )
            self.value = property_value
        elif self.type == ProjectFieldTypes.list.value:
            self.value = list()
            if not isinstance(value, list):
                raise ValueError(f"Value of '{self.name}' must be list.")
            for v in value:  # type: ignore
                self.value.append(self.__class__(self.items, v))  # type: ignore
        else:
            self.value = value

    def __iter__(self):
        """Recursive iteration through potentially nested SectionDataFields
        returns iterator of SectionDataField"""
        if self.type == ProjectFieldTypes.list.value:
            yield self  # First yield self, then nested fields
            # Iterate through list
            for field in self.value:  # type: ignore
                # Iterate through field for recursion
                for f in field:
                    yield f
        elif self.type == ProjectFieldTypes.object.value:
            yield self  # First yield self, then nested fields
            for _, field in self.value.items():  # type: ignore
                for f in field:
                    yield f
        else:
            yield self

    def __len__(self) -> int:
        return len([e for e in self])

    def to_dict(self) -> typing.Union[dict, list, str]:
        if self.type == ProjectFieldTypes.list.value:
            result = list()
            for subfield in self.value:
                if subfield.type == ProjectFieldTypes.enum.value:
                    valid_enums = [choice["value"] for choice in subfield.choices]
                    if subfield.value in valid_enums:
                        result.append({subfield.name: subfield.value})
                else:
                    result.append(subfield.to_dict())
        elif self.type == ProjectFieldTypes.object.value:
            result = dict()
            for name, subfield in self.value.items():
                if subfield.type == ProjectFieldTypes.enum.value:
                    result[name] = {subfield.name: subfield.value}
                result[name] = subfield.to_dict()
        else:
            return self.value
        return result

    def __setattr__(self, __name: str, __value: Any) -> None:
        if __name == "value" and __value is not None:
            if self.type in [
                ProjectFieldTypes.combobox.value,
                ProjectFieldTypes.cvss.value,
                ProjectFieldTypes.string.value,
                ProjectFieldTypes.markdown.value,
            ]:
                if not isinstance(__value, str):
                    raise ValueError(
                        f"'{self.name}' expects a string value (got '{__value}')."
                    )
            elif self.type == ProjectFieldTypes.date.value:
                try:
                    datetime.datetime.strptime(__value, "%Y-%m-%d")
                except (ValueError, TypeError):
                    raise ValueError(
                        f"'{self.name}' expects date in format 2000-01-01 (got '{__value}')."
                    )
            elif self.type == ProjectFieldTypes.enum.value:
                valid_enums = [choice["value"] for choice in self.choices] + [""]
                if __value not in valid_enums:
                    raise ValueError(
                        f"'{__value}' is not an valid enum choice for '{self.name}'."
                    )
            elif self.type == ProjectFieldTypes.list.value:
                if not isinstance(__value, list):
                    raise ValueError(
                        f"Value of '{self.name}' must be list (got '{type(__value)}')."
                    )
                if not all([isinstance(v, self.__class__) for v in __value]):
                    try:
                        # Iterate through list and create objects of self's class
                        if self.items.type in [
                            ProjectFieldTypes.list.value,
                            ProjectFieldTypes.object.value,
                        ]:
                            raise ValueError()

                        new_value = list()
                        for v in __value:
                            if isinstance(v, self.__class__):
                                new_value.append(v)
                            else:
                                new_value.append(self.__class__(self.items, v))
                        __value = new_value
                    except ValueError:
                        raise ValueError(
                            f"Value of '{self.name}' must contain list of {self.__class__.__name__}."
                        )
                types = set([v.type for v in __value])
                if len(types) > 1:
                    raise ValueError(
                        f"Values of '{self.name}' must not contain {self.__class__.__name__}s"
                        f"of multiple types (got {','.join(types)})."
                    )

            elif self.type == ProjectFieldTypes.object.value:
                if not isinstance(__value, dict):
                    raise ValueError(
                        f"Value of '{self.name}' must be dict (got '{type(__value)}')."
                    )
                for k, v in __value.items():
                    if not isinstance(v, self.__class__):
                        raise ValueError(
                            f"Value of '{self.name}' dict values must contain {self.__class__.__name__} "
                            f"(got '{type(v)}' for key '{k}')."
                        )
            elif self.type == ProjectFieldTypes.boolean.value:
                if not isinstance(__value, bool):
                    raise ValueError(
                        f"'{self.name}' expects a boolean value (got '{__value}')."
                    )
            elif self.type == ProjectFieldTypes.number.value:
                if not isinstance(__value, int) and not isinstance(__value, float):
                    raise ValueError(
                        f"'{self.name}' expects int or float (got '{__value}')."
                    )
            elif self.type == ProjectFieldTypes.user.value:
                try:
                    UUID(__value, version=4)
                except (ValueError, AttributeError):
                    raise ValueError(
                        f"'{self.name}' expects v4 uuid (got '{__value}')."
                    )

        return super().__setattr__(__name, __value)


class SectionData(BaseModel):
    field_class = SectionDataField

    def __init__(
        self,
        design_fields: typing.List[ProjectDesignField],
        data_raw: SectionDataRaw,
        force_compatible: bool = True,
    ):
        for design_field in design_fields:
            try:
                value = data_raw.__getattribute__(design_field.name)
            except AttributeError:
                continue
            self.__setattr__(
                design_field.name,
                self.field_class(design_field, value),
            )
        if force_compatible:
            missing_fields = [f for f in data_raw.__dict__ if not hasattr(self, f)]
            if len(missing_fields) > 0:
                raise IncompatibleDesignException(
                    f"Incompatible data and designs: Fields in data but not in design: {','.join(missing_fields)}"
                )

    def __iter__(self):
        """Recursive iteration through cls attributes
        returns FindingDataField"""
        for _, finding_field in vars(self).items():
            for nested_field in finding_field:
                yield nested_field

    def __len__(self) -> int:
        return len([e for e in self])

    def to_dict(self) -> dict:
        result = dict()
        for k, v in deepcopy(vars(self)).items():
            if v.type == ProjectFieldTypes.enum.value:
                valid_enums = [choice["value"] for choice in v.choices]
                if not v.value in valid_enums:
                    # Prevent adding empty enums because server will complain
                    continue
            result[k] = v.to_dict()
        return result


class SectionRaw(BaseModel):
    """
    Attributes:
        project:
        project_type:
        language:
        lock_info:
        template:
        assignee:
        status:
        data:
    """

    project: str = ""
    project_type: str = ""
    language: str = ""
    lock_info: bool = False
    template: str = ""
    assignee: str = ""
    status: str = "in-progress"
    data: SectionDataRaw

    def __init__(self, data, *args, **kwargs):
        if "data" not in data:
            data["data"] = dict()
        super().__init__(data, *args, **kwargs)


class Section(SectionRaw):
    data: SectionData

    def __init__(
        self,
        raw: typing.Union[SectionRaw, typing.Dict],
        project_design: typing.Optional[ProjectDesign] = None,
    ):
        if project_design is None:
            project_design = ProjectDesign()
        if isinstance(raw, dict):
            raw = SectionRaw(raw)

        # Set attributes from SectionRaw
        for attr in typing.get_type_hints(SectionRaw).items():
            self.__setattr__(attr[0], raw.__getattribute__(attr[0]))
        self.data = SectionData(project_design.report_fields, raw.data)



import typing

from reptor.models.ProjectDesign import ProjectDesign
from reptor.models.Section import (
    SectionData,
    SectionDataField,
    SectionDataRaw,
    SectionRaw,
)


class FindingDataRaw(SectionDataRaw):
    """
    Custom finding fields will be added as additional attributes.

    Attributes:
        title:
        cvss:
        summary:
        description:
        precondition:
        impact:
        recommendation:
        short_recommendation:
        references:
        affected_components:
        owasp_top10_2021:
        wstg_category:
        retest_notes:
        retest_status:
        severity:
    """

    title: str = ""
    cvss: str = ""
    summary: str = ""
    description: str = ""
    precondition: str = ""
    impact: str = ""
    recommendation: str = ""
    short_recommendation: str = ""
    references: typing.List[str] = []
    affected_components: typing.List[str] = []
    owasp_top10_2021: str = ""
    wstg_category: str = ""
    retest_notes: str = ""
    retest_status: str = ""
    severity: str = ""


class FindingDataField(SectionDataField):
    ...


class FindingData(SectionData):
    title: FindingDataField
    cvss: FindingDataField
    summary: FindingDataField
    description: FindingDataField
    precondition: FindingDataField
    impact: FindingDataField
    recommendation: FindingDataField
    short_recommendation: FindingDataField
    references: FindingDataField
    affected_components: FindingDataField
    owasp_top10_2021: FindingDataField
    wstg_category: FindingDataField
    retest_notes: FindingDataField
    retest_status: FindingDataField
    severity: FindingDataField

    field_class = FindingDataField


class FindingRaw(SectionRaw):
    data: FindingDataRaw


class Finding(FindingRaw):
    data: FindingData

    def __init__(
        self,
        raw: typing.Union[FindingRaw, typing.Dict],
        project_design: typing.Optional[ProjectDesign] = None,
        force_compatible: bool = True,
    ):
        if project_design is None:
            project_design = ProjectDesign()
        if isinstance(raw, dict):
            raw = FindingRaw(raw)

        # Set attributes from FindingRaw
        for attr in typing.get_type_hints(FindingRaw).items():
            self.__setattr__(attr[0], raw.__getattribute__(attr[0]))
        self.data = FindingData(
            project_design.finding_fields, raw.data, force_compatible=force_compatible
        )

    @classmethod
    def from_translation(
        cls,
        translation: typing.Any,  # translation = FindingTemplateTranslation
        **kwargs,
    ):
        raw = FindingRaw(
            {"language": translation.language, "data": translation.data.to_dict()}
        )
        return cls(raw, **kwargs)



import os
from pathlib import Path

import pytest

from reptor.lib.plugins.TestCaseToolPlugin import TestCaseToolPlugin

from ..Zap import Zap


class TestOwaspZap(TestCaseToolPlugin):
    templates_path = os.path.normpath(Path(os.path.dirname(__file__)) / "../templates")

    @pytest.fixture(autouse=True)
    def setUp(self) -> None:
        Zap.setup_class(
            Path(os.path.dirname(self.templates_path)), skip_user_plugins=True
        )
        self.zap = Zap(reptor=self.reptor)

    def _load_xml_data(self, xml_file):
        self.zap.input_format = "xml"
        filepath = os.path.join(os.path.dirname(__file__), f"./data/{xml_file}")
        with open(filepath, "r") as f:
            self.zap.raw_input = f.read()

    def _load_json_data(self, json_file):
        self.zap.input_format = "json"
        filepath = os.path.join(os.path.dirname(__file__), f"./data/{json_file}")
        with open(filepath, "r") as f:
            self.zap.raw_input = f.read()

    def test_json_parsing(self):
        pass

    def test_xml_parsing(self):
        self._load_xml_data("zap-report.xml")
        self.zap.parse()
        assert len(self.zap.parsed_input) == 2

        assert self.zap.parsed_input[0]["host"] == "localhost"
        assert self.zap.parsed_input[0]["port"] == "443"
        assert self.zap.parsed_input[0]["ssl"] == "true"
        assert self.zap.parsed_input[0]["name"] == "https://localhost"
        assert len(self.zap.parsed_input[0]["alerts"]) == 5

        assert (
            self.zap.parsed_input[0]["alerts"][0]["name"]
            == "Application Error Disclosure"
        )
        assert self.zap.parsed_input[0]["alerts"][0]["riskcode"] == "2"
        assert self.zap.parsed_input[0]["alerts"][0]["confidence"] == "2"
        assert self.zap.parsed_input[0]["alerts"][0]["riskdesc"] == "Medium (Medium)"
        assert self.zap.parsed_input[0]["alerts"][0]["desc"].startswith(
            "This page contains an error/warning"
        )
        assert len(self.zap.parsed_input[0]["alerts"][0]["instances"]) == 34

        assert self.zap.parsed_input[1]["host"] == "localhost"
        assert self.zap.parsed_input[1]["port"] == "80"
        assert self.zap.parsed_input[1]["ssl"] == "false"
        assert self.zap.parsed_input[1]["name"] == "http://localhost"
        assert len(self.zap.parsed_input[1]["alerts"]) == 7

    def test_preprocess_for_template(self):
        self._load_xml_data("zap-report.xml")
        self.zap.parse()
        processed_input = self.zap.preprocess_for_template()
        assert len(processed_input) == 2
        assert list(processed_input.keys()) == [
            "http://localhost (7)",
            "https://localhost (5)",
        ]
        assert "data" in processed_input["http://localhost (7)"]
        assert "data" in processed_input["https://localhost (5)"]

    def test_formatting(self):
        self._load_xml_data("zap-report.xml")
        self.zap.format()
        assert "https://localhost (5)" in self.zap.formatted_input
        assert "http://localhost (7)" in self.zap.formatted_input

        site_details = """| Target | Information |
| :--- | :--- |
| Site | https://localhost |
| Host | localhost |
| Port | 443 |
| SSL ? |  Yes  |"""

        assert site_details in self.zap.formatted_input["https://localhost (5)"]

        csp_issue = """| Target | Information |
| :--- | :--- |
| Risk | Medium (High) |
| Confidence | High |
| Number of Affected Instances | 117 |
| CWE | [693](https://cwe.mitre.org/data/definitions/693.html) |
"""
        assert csp_issue in self.zap.formatted_input["https://localhost (5)"]



import os
from pathlib import Path
from xml.etree import ElementTree

import pytest

from reptor.lib.plugins.TestCaseToolPlugin import TestCaseToolPlugin

from ..models import NiktoScan
from ..Nikto import Nikto


class TestNikto(TestCaseToolPlugin):
    templates_path = os.path.normpath(Path(os.path.dirname(__file__)) / "../templates")
    xml_sample = """<?xml version="1.0" ?>
<!DOCTYPE niktoscans SYSTEM "/var/lib/nikto/docs/nikto.dtd">
<niktoscans>
    <niktoscan hoststest="0" options="-host targets.txt -Format xml -output multiple-nikto.xml" version="2.5.0" scanstart="Thu Jun 29 01:21:02 2023" scanend="Thu Jan  1 02:00:00 1970" scanelapsed=" seconds" nxmlversion="1.2">
    </niktoscan>
</niktoscans>"""

    @pytest.fixture(autouse=True)
    def setUp(self) -> None:
        Nikto.setup_class(
            Path(os.path.dirname(self.templates_path)), skip_user_plugins=True
        )
        self.nikto = Nikto(reptor=self.reptor)

    def _load_xml_data(self, xml_file):
        self.nikto.input_format = "xml"
        filepath = os.path.join(os.path.dirname(__file__), f"./data/{xml_file}")
        with open(filepath, "r") as f:
            self.nikto.raw_input = f.read()

    def _load_json_data(self, json_file):
        self.nikto.input_format = "json"
        filepath = os.path.join(os.path.dirname(__file__), f"./data/{json_file}")
        with open(filepath, "r") as f:
            self.nikto.raw_input = f.read()

    def test_nikto_parse_xml(self):
        self._load_xml_data("multiple-nikto.xml")
        self.nikto.parse()
        assert len(self.nikto.parsed_input) == 2
        assert self.nikto.parsed_input[0].version == "2.5.0"
        assert self.nikto.parsed_input[1].version == "2.5.0"

        assert (
            self.nikto.parsed_input[0].options
            == "-host targets.txt -Format xml -output multiple-nikto.xml"
        )
        assert (
            self.nikto.parsed_input[1].options
            == "-host targets.txt -Format xml -output multiple-nikto.xml"
        )

        assert len(self.nikto.parsed_input[0].scandetails.items) == 2
        assert self.nikto.parsed_input[0].scandetails.errors == "0"
        assert self.nikto.parsed_input[0].scandetails.targetport == "9443"
        assert self.nikto.parsed_input[1].scandetails.targetip == "127.0.0.1"
        assert self.nikto.parsed_input[0].scandetails.targethostname == "localhost"
        assert self.nikto.parsed_input[0].scandetails.targetbanner == ""
        assert self.nikto.parsed_input[0].scandetails.starttime == "2023-06-29 01:21:03"

        assert len(self.nikto.parsed_input[1].scandetails.items) == 151
        assert (
            self.nikto.parsed_input[1].scandetails.targetbanner
            == "Apache/2.4.56 (Debian)"
        )
        assert (
            self.nikto.parsed_input[1].scandetails.targethostname
            == "mutillidae.localhost"
        )
        assert self.nikto.parsed_input[1].scandetails.targetip == "127.0.0.1"
        assert self.nikto.parsed_input[1].scandetails.targetport == "80"
        assert self.nikto.parsed_input[1].scandetails.errors == "0"
        assert self.nikto.parsed_input[1].scandetails.starttime == "2023-06-29 01:21:10"

        assert (
            self.nikto.parsed_input[1].scandetails.items[0].description
            == " Cookie PHPSESSID created without the httponly flag."
        )

    def test_nikto_format(self):
        self._load_xml_data("multiple-nikto.xml")
        self.nikto.format()

        details_table = """| Target | Information |
| :--- | :--- |
| IP | 127.0.0.1 |
| Port | 9443 |
| Hostname | localhost |
| Sitename | http://localhost:9443/ |
| Host Header | localhost |
| Errors | 0 |"""
        assert details_table in self.nikto.formatted_input

        statistics_table = """| Target | Information |
| :--- | :--- |
| Issues Items | 2 |
| Duration | 7 Seconds |
| Total Checks | None |"""
        assert statistics_table in self.nikto.formatted_input

        issues_table = """| Endpoint | Method | Description | References |
| :----- | :--- | :----- | :---- |
| / | GET |  The anti-clickjacking X-Frame-Options header is not present. | https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/X-Frame-Options |
| / | GET |  The X-Content-Type-Options header is not set. This could allow the user agent to render the content of the site in a different fashion to the MIME type. | https://www.netsparker.com/web-vulnerability-scanner/vulnerabilities/missing-content-type-header/ |"""
        assert issues_table in self.nikto.formatted_input

        scan_results = """CMD Options: `-host targets.txt -Format xml -output multiple-nikto.xml`"""
        assert scan_results in self.nikto.formatted_input

        statistics_table_1 = """| Target | Information |
| :--- | :--- |
| Issues Items | 151 |
| Duration | 15 Seconds |
| Total Checks | None |"""
        assert statistics_table_1 in self.nikto.formatted_input

        issues_table_1 = """| Endpoint | Method | Description | References |
| :----- | :--- | :----- | :---- |
| / | GET |  Cookie PHPSESSID created without the httponly flag. | https://developer.mozilla.org/en-US/docs/Web/HTTP/Cookies |
| / | GET |  Cookie showhints created without the httponly flag. | https://developer.mozilla.org/en-US/docs/Web/HTTP/Cookies |
| / | GET |  Retrieved x-powered-by header | None |"""
        assert issues_table_1 in self.nikto.formatted_input

    def test_nikto_scan(self):
        root = ElementTree.fromstring(self.xml_sample)
        nikto_scan = NiktoScan()
        nikto_scan.parse(root[0])
        assert (
            "-host targets.txt -Format xml -output multiple-nikto.xml"
            == nikto_scan.options
        )
        assert "2.5.0" == nikto_scan.version



import pathlib
import shutil

import reptor.settings as settings
import reptor.subcommands as subcommands
from reptor.lib.plugins.Base import Base
from reptor.utils.table import make_table


class Plugins(Base):
    """
    Use this plugin to list your plugins.

    Search for plugins based on tags, name or author

    Create a new plugin from a template for the community
    or yourself.

    # Arguments

    * --search    Allows you to search for a specific plugin by name and tag
    * --copy PLUGINNAME      Copies a plugin to your local folder for development
    * --new PLUGINNAME  Creates a new plugin based off a template and your input
    * --verbose   Provides more information about a plugin

    # Developer Notes
    You can modify the `_list` method to change the output for search as
    well as the default output.
    """

    meta = {
        "name": "Plugins",
        "summary": "Allows plugin management & development",
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.search = kwargs.get("search")
        self.new_plugin_name = kwargs.get("new_plugin_name")
        self.copy_plugin_name = kwargs.get("copy_plugin_name")
        self.verbose = kwargs.get("verbose")

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath)

        action_group = parser.add_mutually_exclusive_group()
        action_group.title = "action_group"
        action_group.add_argument(
            "-search",
            "--search",
            metavar="SEARCHTERM",
            action="store",
            dest="search",
            const="",
            nargs="?",
            help="Search for term",
        )
        action_group.add_argument(
            "-new",
            "--new",
            metavar="PLUGINNAME",
            nargs="?",
            dest="new_plugin_name",
            const="",
            help="Create a new plugin",
        )
        action_group.add_argument(
            "-copy",
            "--copy",
            metavar="PLUGINNAME",
            action="store",
            dest="copy_plugin_name",
            help="Copy plugin to home directory",
        )

    def _list(self, plugins):
        if self.verbose:
            table = make_table(
                [
                    "Name",
                    "Short Help",
                    "Space",
                    "Overwrites",
                    "Author",
                    "Version",
                    "Tags",
                ]
            )
        else:
            table = make_table(
                [
                    "Name",
                    "Short Help",
                    "Tags",
                ]
            )

        for tool in plugins:
            color = "blue"
            tool_name = f"[{color}]{tool.name}[/{color}]"

            overwritten_plugin = tool.get_overwritten_plugin()
            overwrites_name = ""
            if overwritten_plugin:
                overwrites_name = f"[red]{overwritten_plugin.name}({overwritten_plugin.category})[/red]"
                tool_name = f"[red]{tool.name}({tool.category})\nOverwrites: {overwritten_plugin.category}[/red]"
                color = "red"

            if self.verbose:
                table.add_row(
                    f"[{color}]{tool.name}[/{color}]",
                    f"[{color}]{tool.summary}[/{color}]",
                    f"[{color}]{tool.category}[/{color}]",
                    overwrites_name,
                    f"[{color}]{tool.author}[/{color}]",
                    f"[{color}]{tool.version}[/{color}]",
                    f"[{color}]{','.join(tool.tags)}[/{color}]",
                )
            else:
                table.add_row(
                    f"{tool_name}",
                    f"[{color}]{tool.summary}[/{color}]",
                    f"[{color}]{','.join(tool.tags)}[/{color}]",
                )

        self.console.print(table)

    def _search(self):
        """Searches plugins"""
        plugins = list()
        for _, group_plugins in subcommands.SUBCOMMANDS_GROUPS.items():
            plugins.extend(group_plugins[1])
        if self.search:
            self.console.print(f"\nSearching for: [red]{self.search}[/red]\n")
            results = list()
            for plugin in plugins:
                if self.search in plugin.tags:
                    results.append(plugin)
                    continue

                if self.search in plugin.name:
                    results.append(plugin)
                    continue
        else:
            results = plugins

        self._list(results)

    def _create_new_plugin(self):
        """Goes through a few questions to generate a new plugin.

        The user must at least answer the plugin Name.

        It should always be a directory based plugin, because of templates.

        The user can then go on and develop their own plugin.
        """

        # Todo: Finalise this to make it as easy as possible for new plugins to be created by anyone
        # think Django's manage.py startapp or npm init

        introduction = """
        Please answer the following questions.

        Based on the answers we will create a raw plugin for you to work on.
        You will find the new plugin in your ~/.sysrepter/plugins folder.

        Let's get started...
        """

        self.reptor.console.print(introduction)

        if self.new_plugin_name:
            plugin_name = self.new_plugin_name.strip().split(" ")[0]
            print(f"plugin Name: {plugin_name}")
        else:
            plugin_name = (
                (input("plugin Name (No spaces, try to use the tool name): "))
                .strip()
                .split(" ")[0]
            )

        plugin_name.strip().split(" ")[0]

        author = input("Author Name: ")[:25] or "Unknown"
        tags = input("Tags (only first: 5 Tags), i.e owasp,web,scanner: ").split(",")[
            :5
        ]

        # Create the folder
        new_plugin_folder = pathlib.Path(settings.PLUGIN_DIRS_USER / plugin_name)

        try:
            new_plugin_folder.mkdir(parents=True)
        except FileExistsError:
            self.highlight(
                "A plugin with this name already exists in your home directory."
            )
            overwrite = input("Do you want to continue? [N/y]: ") or "n"

            if overwrite[0].lower() != "y":
                raise AssertionError("Aborting...")

        shutil.copytree(
            settings.PLUGIN_TOOLBASE_TEMPLATE_FOLDER,
            new_plugin_folder / "",
            dirs_exist_ok=True,
        )

        # Now rename some stuff and replace some placeholders
        new_plugin_file = new_plugin_folder / f"{plugin_name.capitalize()}.py"
        pathlib.Path(new_plugin_folder / "Toolbase.py").rename(new_plugin_file)

        with open(new_plugin_file, "r+") as f:
            contents = f.read()
            contents = contents.replace("MYMODULENAME", plugin_name.capitalize())
            contents = contents.replace("AUTHOR_NAME", author)
            contents = contents.replace("TAGS_LIST", ",".join(tags))

            f.seek(0)
            f.write(contents)
            f.truncate()

        self.log.success(f"New plugin created. Happy coding! ({new_plugin_folder})")

    def _copy_plugin(self, dest=settings.PLUGIN_DIRS_USER):
        # Check if plugin exists and get its path
        try:
            plugin = pathlib.Path(
                self.reptor.plugin_manager.LOADED_PLUGINS[
                    self.copy_plugin_name
                ].__file__
            )
        except KeyError:
            raise ValueError(f"Plugin '{self.copy_plugin_name}' does not exist.")

        # Copy plugin

        dest = dest / plugin.parent.name
        self.log.display(f'Trying to copy "{plugin.parent}" to "{dest}"')
        shutil.copytree(plugin.parent, dest)
        self.log.success(f"Copied successfully. ({dest})")

    def run(self):
        if self.new_plugin_name is not None:
            self._create_new_plugin()
        elif self.copy_plugin_name is not None:
            self._copy_plugin()
        else:
            self._search()


loader = Plugins



import datetime
import typing

from reptor.lib.plugins.ModelBase import ModelBase


class Item(ModelBase):
    method: str = "GET"
    description: str
    uri: str
    namelink: str
    iplink: str
    references: str

    endpoint: str = "/"

    def parse(self, data):
        description = data.find("description").text
        self.description = description
        if ": " in description:
            description_data = description.split(":")
            self.endpoint = description_data[0]
            self.description = description_data[1]

        self.uri = data.find("uri").text
        self.namelink = data.find("namelink").text
        self.iplink = data.find("iplink").text
        self.references = data.find("references").text
        self.method = data.get("method")


class ScanDetails(ModelBase):
    targetip: str
    targethostname: str
    targetport: str
    targetbanner: str
    sitename: str
    siteip: str
    hostheader: str
    errors: str
    checks: str

    items: typing.List[Item] = list()

    def parse(self, data):
        self.targethostname = data.get("targethostname")
        self.targetport = data.get("targetport")
        self.targetbanner = data.get("targetbanner")
        self.sitename = data.get("sitename")
        self.hostheader = data.get("hostheader")
        self.siteip = data.get("siteip")
        self.targetip = data.get("targetip")
        self.starttime = data.get("starttime")
        self.errors = data.get("errors")

    def append_item(self, item: Item):
        if not self.items:
            self.items = list()

        self.items.append(item)

    def set_items(self, items: typing.List[Item]):
        self.items = items


class Statistics(ModelBase):
    elapsed: int
    errors: int
    checks: int
    itemsfound: int
    itemstested: int
    starttime: datetime.datetime
    endtime: datetime.datetime

    def parse(self, data):
        self.elapsed = data.get("elapsed")
        self.itemsfound = data.get("itemsfound")
        self.endtime = data.get("endtime")
        self.itemstested = data.get("itemstested")
        self.checks = data.get("checks")
        self.errors = data.get("errors")


class NiktoScan(ModelBase):
    options: str
    version: str
    scandetails: ScanDetails
    statistics: Statistics

    def parse(self, data):
        self.options = data.get("options")
        self.version = data.get("version")



import logging
import pathlib
import re
import sys
import typing

from datetime import datetime
from logging import LogRecord
from logging.handlers import RotatingFileHandler

from termcolor import colored
from rich.text import Text
from rich.logging import RichHandler
from rich.markup import escape as rich_escape

import reptor.settings as settings

from reptor.lib.console import reptor_console

# This code was heavily copied from https://github.com/mpgn/CrackMapExec/blob/master/cme/logger.py
# However it was also modified to our needs
# See NOTICE for the CrackMapExec License


class ReptorAdapter(logging.LoggerAdapter):
    def __init__(self, extra=None):
        logging.basicConfig(
            format="%(message)s",
            datefmt="[%X]",
            handlers=[
                RichHandler(
                    console=reptor_console,
                    rich_tracebacks=True,
                    tracebacks_show_locals=False,
                )
            ],
        )
        self.logger = logging.getLogger("reptor")
        self.extra = extra
        self.output_file = None

    def _print(self, msg, color="white", *args, **kwargs):
        msg, kwargs = self._format(
            f"{colored(f'{msg}', color, attrs=['bold'])}", kwargs
        )
        text = Text.from_ansi(msg)
        reptor_console.print(text, *args, **kwargs)
        self._log_console_to_file(text, *args, **kwargs)

    def _format(self, msg, *args, **kwargs):
        """
        Format msg for output if needed
        """
        return f"{msg}", kwargs

    def escape(self, *args, **kwargs) -> str:
        return rich_escape(*args, **kwargs)

    def display(self, msg, color="blue", *args, **kwargs):
        """Prints a blue message in console

        Args:
            msg (_type_): _description_
            color (str, optional): _description_. Defaults to "blue".
        """
        self._print(msg, color, *args, **kwargs)

    def fail(self, msg, color="red", *args, **kwargs):
        """Prints a red message

        Args:
            msg (_type_): _description_
            color (str, optional): _description_. Defaults to "red".
        """
        self._print(msg, color, *args, **kwargs)

    def fail_with_exit(self, msg, color="red", *args, **kwargs):
        """Prints a red message and calls sys.exit with a status code of 1

        Args:
            msg (_type_): _description_
            color (str, optional): _description_. Defaults to "red".
        """
        self._print(msg, color, *args, **kwargs)
        exit(1)

    def success(self, msg, color="green", *args, **kwargs):
        """Prints a green message

        Args:
            msg (_type_): _description_
            color (str, optional): _description_. Defaults to "green".
        """
        self._print(msg, color, *args, **kwargs)

    def highlight(self, msg, *args, **kwargs):
        """
        Prints a completely yellow highlighted message to the user
        """
        msg, kwargs = self._format(
            f"{colored(msg, 'yellow', attrs=['bold'])}", kwargs)
        text = Text.from_ansi(msg)
        reptor_console.print(text, *args, **kwargs)
        self._log_console_to_file(text, *args, **kwargs)

    def _log_console_to_file(self, text, *args, **kwargs):
        """
        If debug or info logging is not enabled, we still want display/success/fail logged to the file specified,
        so we create a custom LogRecord and pass it to all the additional handlers (which will be all the file handlers
        """
        if self.logger.getEffectiveLevel() >= logging.INFO:
            # will be 0 if it's just the console output, so only do this if we actually have file loggers
            if len(self.logger.handlers):
                try:
                    for handler in self.logger.handlers:
                        handler.handle(
                            LogRecord(
                                "reptor",
                                20,
                                "",
                                kwargs,  # type: ignore
                                msg=text,
                                args=args,
                                exc_info=None,
                            )
                        )
                except Exception as e:
                    self.logger.error(
                        f"Issue while trying to custom print handler: {e}"
                    )
        else:
            self.logger.info(text)

    def add_file_log(self, log_file: typing.Union[pathlib.Path, None] = None):
        file_formatter = TermEscapeCodeFormatter(
            "%(asctime)s - %(levelname)s - %(message)s"
        )
        output_file = self.init_log_file() if log_file is None else log_file

        file_handler = RotatingFileHandler(output_file, maxBytes=100000)

        with open(output_file, "a", encoding="utf-8") as f:
            f.write(
                "\n[%s]> %s\n\n"
                % (datetime.now().strftime("%d-%m-%Y %H:%M:%S"), " ".join(sys.argv))
            )

        file_handler.setFormatter(file_formatter)
        self.logger.addHandler(file_handler)
        self.logger.debug(f"Added file handler: {file_handler}")

    @staticmethod
    def init_log_file():
        """Creates parent folder structure in home directory of user
        Sets the current Logfile name

        Returns:
            pathlib.Path: Full pathlib Path to log FILE
        """
        # Date Based
        settings.LOG_FOLDER.mkdir(parents=True, exist_ok=True)
        return settings.LOG_FOLDER / "reptor.log"


class TermEscapeCodeFormatter(logging.Formatter):
    """A class to strip the escape codes for logging to files"""

    def __init__(self, fmt=None, datefmt=None, style="%", validate=True):
        super().__init__(fmt, datefmt, style, validate)  # type: ignore

    def format(self, record):
        escape_re = re.compile(r"\x1b\[[0-9;]*m")
        record.msg = re.sub(escape_re, "", str(record.msg))
        return super().format(record)


reptor_logger = ReptorAdapter()



import typing

from reptor.lib.plugins.ToolBase import ToolBase


class Sslyze(ToolBase):
    """
    target="app1.example.com:443{127.0.0.1} app2.example.com:443{127.0.0.2}"
    sslyze --sslv2 --sslv3 --tlsv1 --tlsv1_1 --tlsv1_2 --tlsv1_3 --certinfo --reneg --http_get --hide_rejected_ciphers --compression --heartbleed --openssl_ccs --fallback --robot "$target" --json_out=- | tee sslyze.txt | reptor sslyze

    # Format and upload
    cat sslyze_result.txt | reptor ssylyze -c upload
    """

    meta = {
        "author": "Syslifters",
        "name": "Sslyze",
        "version": "1.0",
        "license": "MIT",
        "tags": ["web", "ssl"],
        "summary": "format sslyze JSON output",
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.note_icon = "🔒"
        self.input_format = "json"

    weak_ciphers = [
        "DHE-DSS-DES-CBC3-SHA",
        "DHE-DSS-AES128-SHA",
        "DHE-DSS-AES128-SHA256",
        "DHE-DSS-AES256-SHA",
        "DHE-DSS-AES256-SHA256",
        "DHE-DSS-CAMELLIA128-SHA",
        "DHE-DSS-CAMELLIA128-SHA256",
        "DHE-DSS-CAMELLIA256-SHA",
        "DHE-DSS-CAMELLIA256-SHA256",
        "DHE-DSS-SEED-SHA",
        "DHE-PSK-3DES-EDE-CBC-SHA",
        "DHE-PSK-AES128-CBC-SHA",
        "DHE-PSK-AES128-CBC-SHA256",
        "DHE-PSK-AES256-CBC-SHA",
        "DHE-PSK-AES256-CBC-SHA384",
        "DHE-PSK-CAMELLIA128-SHA256",
        "DHE-PSK-CAMELLIA256-SHA384",
        "DHE-RSA-DES-CBC3-SHA",
        "DHE-RSA-AES128-SHA",
        "DHE-RSA-AES128-SHA256",
        "DHE-RSA-AES256-SHA",
        "DHE-RSA-AES256-SHA256",
        "DHE-RSA-CAMELLIA128-SHA",
        "DHE-RSA-CAMELLIA128-SHA256",
        "DHE-RSA-CAMELLIA256-SHA",
        "DHE-RSA-CAMELLIA256-SHA256",
        "DHE-RSA-SEED-SHA",
        "ECDHE-ECDSA-DES-CBC3-SHA",
        "ECDHE-ECDSA-AES128-SHA",
        "ECDHE-ECDSA-AES128-SHA256",
        "ECDHE-ECDSA-AES256-SHA",
        "ECDHE-ECDSA-AES256-SHA384",
        "ECDHE-ECDSA-CAMELLIA128-SHA256",
        "ECDHE-ECDSA-CAMELLIA256-SHA384",
        "ECDHE-PSK-3DES-EDE-CBC-SHA",
        "ECDHE-PSK-AES128-CBC-SHA",
        "ECDHE-PSK-AES128-CBC-SHA256",
        "ECDHE-PSK-AES256-CBC-SHA",
        "ECDHE-PSK-AES256-CBC-SHA384",
        "ECDHE-PSK-CAMELLIA128-SHA256",
        "ECDHE-PSK-CAMELLIA256-SHA384",
        "ECDHE-RSA-DES-CBC3-SHA",
        "ECDHE-RSA-AES128-SHA",
        "ECDHE-RSA-AES128-SHA256",
        "ECDHE-RSA-AES256-SHA",
        "ECDHE-RSA-AES256-SHA384",
        "ECDHE-RSA-CAMELLIA128-SHA256",
        "ECDHE-RSA-CAMELLIA256-SHA384",
        "PSK-3DES-EDE-CBC-SHA",
        "PSK-AES128-CBC-SHA",
        "PSK-AES128-CBC-SHA256",
        "PSK-AES128-CCM",
        "PSK-AES128-CCM8",
        "PSK-AES128-GCM-SHA256",
        "PSK-AES256-CBC-SHA",
        "PSK-AES256-CBC-SHA384",
        "PSK-AES256-CCM",
        "PSK-AES256-CCM8",
        "PSK-AES256-GCM-SHA384",
        "PSK-CAMELLIA128-SHA256",
        "PSK-CAMELLIA256-SHA384",
        "PSK-CHACHA20-POLY1305",
        "RSA-PSK-3DES-EDE-CBC-SHA",
        "RSA-PSK-AES128-CBC-SHA",
        "RSA-PSK-AES128-CBC-SHA256",
        "RSA-PSK-AES128-GCM-SHA256",
        "RSA-PSK-AES256-CBC-SHA",
        "RSA-PSK-AES256-CBC-SHA384",
        "RSA-PSK-AES256-GCM-SHA384",
        "RSA-PSK-CAMELLIA128-SHA256",
        "RSA-PSK-CAMELLIA256-SHA384",
        "RSA-PSK-CHACHA20-POLY1305",
        "DES-CBC3-SHA",
        "AES128-SHA",
        "AES128-SHA256",
        "AES128-CCM",
        "AES128-CCM8",
        "AES128-GCM-SHA256",
        "AES256-SHA",
        "AES256-SHA256",
        "AES256-CCM",
        "AES256-CCM8",
        "AES256-GCM-SHA384",
        "CAMELLIA128-SHA",
        "CAMELLIA128-SHA256",
        "CAMELLIA256-SHA",
        "CAMELLIA256-SHA256",
        "IDEA-CBC-SHA",
        "SEED-SHA",
        "SRP-DSS-3DES-EDE-CBC-SHA",
        "SRP-DSS-AES-128-CBC-SHA",
        "SRP-DSS-AES-256-CBC-SHA",
        "SRP-RSA-3DES-EDE-CBC-SHA",
        "SRP-RSA-AES-128-CBC-SHA",
        "SRP-RSA-AES-256-CBC-SHA",
        "SRP-3DES-EDE-CBC-SHA",
        "SRP-AES-128-CBC-SHA",
        "SRP-AES-256-CBC-SHA",
    ]
    insecure_ciphers = [
        "ADH-DES-CBC3-SHA",
        "ADH-AES128-SHA",
        "ADH-AES128-SHA256",
        "ADH-AES128-GCM-SHA256",
        "ADH-AES256-SHA",
        "ADH-AES256-SHA256",
        "ADH-AES256-GCM-SHA384",
        "ADH-CAMELLIA128-SHA",
        "ADH-CAMELLIA128-SHA256",
        "ADH-CAMELLIA256-SHA",
        "ADH-CAMELLIA256-SHA256",
        "ADH-SEED-SHA",
        "DHE-PSK-NULL-SHA",
        "DHE-PSK-NULL-SHA256",
        "DHE-PSK-NULL-SHA384",
        "AECDH-DES-CBC3-SHA",
        "AECDH-AES128-SHA",
        "AECDH-AES256-SHA",
        "AECDH-NULL-SHA",
        "ECDHE-ECDSA-NULL-SHA",
        "ECDHE-PSK-NULL-SHA",
        "ECDHE-PSK-NULL-SHA256",
        "ECDHE-PSK-NULL-SHA384",
        "ECDHE-RSA-NULL-SHA",
        "PSK-NULL-SHA",
        "PSK-NULL-SHA256",
        "PSK-NULL-SHA384",
        "RSA-PSK-NULL-SHA",
        "RSA-PSK-NULL-SHA256",
        "RSA-PSK-NULL-SHA384",
        "NULL-MD5",
        "NULL-SHA",
        "NULL-SHA256",
    ]

    def get_weak_ciphers(self, target):
        result_protocols = dict()
        for protocol, protocol_data in target["commands_results"].items():
            if len(protocol_data.get("accepted_cipher_list", list())) > 0:
                result_protocols[protocol] = dict()
                weak_ciphers = list(
                    filter(
                        None,
                        [
                            c["openssl_name"]
                            if c["openssl_name"] in self.weak_ciphers
                            else None
                            for c in protocol_data["accepted_cipher_list"]
                        ],
                    )
                )
                if weak_ciphers:
                    result_protocols[protocol]["weak_ciphers"] = weak_ciphers
                insecure_ciphers = list(
                    filter(
                        None,
                        [
                            c["openssl_name"]
                            if c["openssl_name"] in self.insecure_ciphers
                            else None
                            for c in protocol_data["accepted_cipher_list"]
                        ],
                    )
                )
                if insecure_ciphers:
                    result_protocols[protocol]["insecure_ciphers"] = insecure_ciphers
        return result_protocols

    def get_certinfo(self, target):
        result_certinfo = dict()
        certinfo = target.get("commands_results", dict()).get("certinfo")
        if certinfo is None:
            return result_certinfo
        result_certinfo["certificate_matches_hostname"] = certinfo.get(
            "certificate_matches_hostname"
        )
        result_certinfo["has_sha1_in_certificate_chain"] = certinfo.get(
            "has_sha1_in_certificate_chain"
        )
        result_certinfo["certificate_untrusted"] = list(
            filter(
                None,
                [
                    store["trust_store"]["name"]
                    if not store["is_certificate_trusted"]
                    else None
                    for store in certinfo.get("path_validation_result_list") or list()
                ],
            )
        )
        return result_certinfo

    def get_vulnerabilities(self, target):
        result_vulnerabilities = dict()
        commands_results = target.get("commands_results")
        if commands_results is None:
            return result_vulnerabilities
        result_vulnerabilities["heartbleed"] = commands_results.get(
            "heartbleed", dict()
        ).get("is_vulnerable_to_heartbleed")
        result_vulnerabilities["openssl_ccs"] = commands_results.get(
            "openssl_ccs", dict()
        ).get("is_vulnerable_to_ccs_injection")
        robot = commands_results.get("robot", dict()).get("robot_result_enum")
        result_vulnerabilities["robot"] = (
            robot if "NOT_VULNERABLE" not in robot else False
        )
        return result_vulnerabilities

    def get_misconfigurations(self, target):
        result_misconfigs = dict()
        commands_results = target.get("commands_results")
        if commands_results is None:
            return result_misconfigs
        result_misconfigs["compression"] = (
            commands_results.get("compression", dict()).get("compression_name")
            is not None
        )
        result_misconfigs["downgrade"] = (
            commands_results.get("fallback", dict()).get("supports_fallback_scsv", True)
            is not True
        )
        result_misconfigs["client_renegotiation"] = commands_results.get(
            "reneg", dict()
        ).get("accepts_client_renegotiation")
        result_misconfigs["no_secure_renegotiation"] = (
            commands_results.get("reneg", dict()).get(
                "supports_secure_renegotiation", True
            )
            is not True
        )

        return result_misconfigs

    def get_server_info(self, target):
        result_server_info = dict()
        server_info = target.get("server_info")
        result_server_info["hostname"] = server_info["hostname"]
        result_server_info["port"] = str(server_info["port"])
        result_server_info["ip_address"] = server_info["ip_address"]
        return result_server_info

    def preprocess_for_template(self):
        data = list()
        if not isinstance(self.parsed_input, dict):
            return None
        for target in self.parsed_input.get("accepted_targets", list()):
            target_data = self.get_server_info(target)
            target_data["protocols"] = self.get_weak_ciphers(target)
            target_data["has_weak_ciphers"] = False
            if any(
                [
                    v.get("weak_ciphers", list()) + v.get("insecure_ciphers", list())
                    for k, v in target_data["protocols"].items()
                ]
            ):
                target_data["has_weak_ciphers"] = True
            target_data["certinfo"] = self.get_certinfo(target)
            target_data["vulnerabilities"] = self.get_vulnerabilities(target)
            target_data["has_vulnerabilities"] = False
            if any([v for k, v in target_data["vulnerabilities"].items()]):
                target_data["has_vulnerabilities"] = True
            target_data["misconfigurations"] = self.get_misconfigurations(target)

            data.append(target_data)
        if self.template in [
            "protocols",
            "certinfo",
            "vulnerabilities",
            "misconfigurations",
            "weak_ciphers",
        ]:
            if len(data) > 1:
                self.log.warning(
                    "sslyze output contains more than one target. Taking the first one."
                )
            return {"target": data[0]}
        return {"data": data}

    def finding_weak_ciphers(self):
        finding_context = self.preprocess_for_template()
        if finding_context is None:
            return None
        if any(
            [
                target.get("has_weak_ciphers")
                for target in finding_context.get("data", [])
            ]
        ):
            # Add affected components
            finding_context["affected_components"] = [
                f'{t["hostname"]}:{t["port"]} ({t["ip_address"]})'
                for t in finding_context["data"]
            ]
            return finding_context
        return None


loader = Sslyze



from reptor.models.Base import BaseModel


class Note(BaseModel):
    """
    Attributes:
        lock_info:
        title:
        text:
        checked:
        icon_emoji:
        status_emoji:
        order:
        parent:
    """

    lock_info: bool = False
    title: str = ""
    text: str = ""
    checked: bool = False
    icon_emoji: str = ""
    status_emoji: str = ""
    order: int = 0
    parent: str = ""



import typing

from reptor.models.Base import BaseModel
from reptor.models.Finding import Finding
from reptor.models.Section import Section
from reptor.models.User import User


class Project(BaseModel):
    """
    Attributes:
        name:
        project_type:
        language:
        tags:
        readonly:
        source:
        copy_of:
        members:
    """

    name: str = ""

    project_type: str = ""  # is the project design id

    language: str = ""
    tags: typing.List[str] = []
    readonly: bool = False
    source: str = ""
    copy_of: str = ""
    override_finding_order: bool = False
    members: typing.List[User] = []
    imported_members: typing.List[User] = []
    details: str = ""
    notes: str = ""
    images: str = ""
    findings: typing.List[Finding] = []
    sections: typing.List[Section] = []




from .NotesAPI import NotesAPI
from .ProjectDesignsAPI import ProjectDesignsAPI
from .ProjectsAPI import ProjectsAPI
from .TemplatesAPI import TemplatesAPI


class APIManager:
    _notes: NotesAPI = None
    _projects: ProjectsAPI = None
    _project_designs: ProjectDesignsAPI = None
    _templates: TemplatesAPI = None

    def __init__(self, **kwargs) -> None:
        self._reptor = kwargs["reptor"]
        self._project_id = self._reptor.get_active_project_id()

    @property
    def notes(self) -> NotesAPI:
        if not self._notes:
            self._notes = NotesAPI(reptor=self._reptor, project_id=self._project_id)
        return self._notes

    @property
    def projects(self) -> ProjectsAPI:
        if not self._projects:
            self._projects = ProjectsAPI(
                reptor=self._reptor, project_id=self._project_id
            )
        return self._projects

    def switch_project(self, new_project_id) -> None:
        self._project_id = new_project_id
        self._projects = None
        self._notes = None
        self._project_designs = None
        self._templates = None

    @property
    def project_designs(self) -> ProjectDesignsAPI:
        if not self._project_designs:
            project_design_id = self.projects.project.project_type
            self._project_designs = ProjectDesignsAPI(
                reptor=self._reptor, project_design_id=project_design_id
            )
        return self._project_designs

    @property
    def templates(self) -> TemplatesAPI:
        if not self._templates:
            self._templates = TemplatesAPI(
                reptor=self._reptor, project_id=self._project_id
            )
        return self._templates



from enum import StrEnum


class OWASP_TOP10_2021(StrEnum):
    A01_2021: str = "A01_2021"  #'A01:2021 - Broken Access Control'
    A02_2021: str = "A02_2021"  #'A02:2021 - Cryptographic Failures'
    A03_2021: str = "A03_2021"  #'A03:2021 - Injection'
    A04_2021: str = "A04_2021"  #'A04:2021 - Insecure Design'
    A05_2021: str = "A05_2021"  #'A05:2021 - Security Misconfiguration'
    A06_2021: str = "A06_2021"  #'A06:2021 - Vulnerable and Outdated Components'
    A07_2021: str = "A07_2021"  #'A07:2021 - Identification and Authentication Failures'
    A08_2021: str = "A08_2021"  #'A08:2021 - Software and Data Integrity Failures'
    A09_2021: str = "A09_2021"  #'A09:2021 - Security Logging and Monitoring Failures'
    A10_2021: str = "A10_2021"  #'A10:2021 - Server-Side Request Forgery (SSRF)'


class WSTG_CATEGORY(StrEnum):
    INFO: str = "INFO"  # Information Gathering
    CONF: str = "CONF"  # Configuration and Deployment Management
    IDNT: str = "IDNT"  # Identity Management
    ATHN: str = "ATHN"  # Authentication
    ATHZ: str = "ATHZ"  # Authorization
    SESS: str = "SESS"  # Session Management
    INPV: str = "INPV"  # Input Validation
    ERRH: str = "ERRH"  # Error Handling
    CRYP: str = "CRYP"  # Weak Cryptography
    BUSL: str = "BUSL"  # Business Logic
    CLNT: str = "CLNT"  # Client-side Testing
    APIT: str = "APIT"  # API Testing


class RETEST_STATUS(StrEnum):
    OPEN: str = "open"
    RESOLVED: str = "resolved"
    PARTIALLY_RESOLVED: str = "partial"
    CHANGED: str = "changed"
    ACCEPTED: str = "accepted"
    NEW: str = "new"
